'
' Amicus18 GPS Receiver Shield Demo written by Les Johnson, 14th August 2011
' Modified by Alberto Freixanet 17th August 2017.
' Compiled with the PDS version 3.6.0.2.
'
' Receive GPS data and parse it into date, time, and position
' Display the information on the serial terminal
'
' Compile the HEX for the Amicus18 Board for FOSC = 80 Mhz.
    $define _AMicus18_80Mhz_
'###############################################################################
' Compile the HEX for the Amicus18 Board for FOSC = 64 Mhz.
'    $define _AMicus18_64Mhz_
'###############################################################################
$if !_defined(_AMicus18_80Mhz_) And !_defined(_AMicus18_64Mhz_)
    $error "An Amicus18 board must be defined"
$endif
'###############################################################################
' Choose the Bootloader you are using for the Amicus18 Board (real hardware).
' To Define the config fuses.
' CHOOSE ONLY ONE.
'    $define _AmicusBootloader_
    $define _AGV_LSM_BootloaderV41_
'###############################################################################
' All FILES LIST for this PROJECT:
' GPS_Shield_Demo.bas (main code)
'
' SM_GPS_PIC18F25K20.Inc
' SYS_18FXXK20.inc
' DeclareLocalVars.inc
' HRSOut_K40.Inc
' GPS_WDG_ZX4120.Inc

'###############################################################################
'*******************************************************************************
'* Name    : Declare the PIC used.                                             *
'* Purpose : Declare all datas of the used PIC                                 *
'* Notes   : Change the file for a new pIC.                                    *
'******************************************************************************* 
'    Include "D:\MyIncFiles\SM_GPS_PIC18F25K20.Inc"
    Include "INC\SM_GPS_PIC18F25K20.Inc"
'###############################################################################
' THE PROJECT STARTS HERE:
'###############################################################################
'
'###############################################################################
    Declare Hserial_Baud = 115200   ' Set baud rate to 115200
    Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
    Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode
    Declare Hserial_Clear = On      ' Enable Error clearing on received characters    
'###############################################################################
'    Include "D:\MyIncFiles\HRSOut_K40.inc"
    Include "INC\HRSOut_K40.inc"    
'-------------------------------------------------------------------------------
    ' Enabled this line to enable the Synchronized Operation of reading the files every second. 
    $define _SynchronizedOperation_
'-------------------------------------------------------------------------------
' Define the name of the files to read.
    $define _ReadGPGSA_
    $define _ReadGPGSV_
    $define _ReadGPVTG_
    '---------------------------------------------------------------------------
' Define the name of the buffers to print.
    $define _PrintGPGGA_
    $define _PrintGPGSA_    
    $define _PrintGPGSV_ 
    $define _PrintGPRMC_
    $define _PrintGPVTG_
    '---------------------------------------------------------------------------
' Define the calculations to do.
    $define _Altitude_
    $define _LatLonToLocator_
    $define _LocatorToLatLon_
    $define _CalcDistance_
    '---------------------------------------------------------------------------
' Define more options.
'    $define _GPS_Configuration_

    '---------------------------------------------------------------------------
' Define the reset before send a new data to the terminal (Yes or No).
    $define _ResetAmicus18Terminal_
'-------------------------------------------------------------------------------
'    Include "D:\MyIncFiles\GPS_WDG_ZX4120.inc"      ' Load the GPS routines into the program
    Include "INC\GPS_WDG_ZX4120.inc"                ' Load the GPS routines into the program
'------------------------------------------------------------------------------
' Declare some variables for the main code.
    Dim Distance As Dword
    Dim AntBearing As Word
    Dim LAT_Dd2 As Float
    Dim LON_Dd2 As Float
    Dim LATi_Dd As Float       
    Dim LONg_Dd As Float
    Dim MyQTHStrg As String * 6
'===============================================================================
' Check the Xtal of Code.
    $if _xtal = 80
    HRSOut "XTAL = 80Mhz",13,10   
    $endif
    $if _xtal = 64
    HRSOut "XTAL = 64Mhz",13,10    
    $endif    
    HRSOut "GPS NMEA Demo:",13,10
    HRSOut 13,10     
    DelayMS 3000
'###############################################################################
INIT_GPS:
' Something to do?
' Send data with delay of 50mS between files.

    GPS_Heading = 0
    GPS_SpeedKm = 0
    GPS_Altitude = 0
    GPS_SatsInView = 0
'===============================================================================
Main:      
    While 1 = 1                     ' Create an infinite loop
        GPS_Parse()                 ' Gather the GPS serial data and parse it
        ' Display the GPS information on the serial terminal
        HRSOut "Date: ", Dec2 GPS_Day, ":", Dec2 GPS_Month, ":", Dec2 GPS_Year, 13,10,_
               "Time: ", Dec2 GPS_Hours, ":", Dec2 GPS_Minutes, ":", Dec2 GPS_Seconds, " UTC", 13,10

        Select GPS_FixType          ' Has a GPS fix has occured?
            Case > 0                ' Yes. So display the results
                ' Fix = 0 => invalid
                ' Fix = 1 => GPS fix (SPS)
                ' Fix = 2 => DGPS fix
                ' Fix = 3 => PPS fix
                HRSOut "Fix type: ", Dec GPS_FixType,13,10 
                HRSOut "Sats in view: ", Dec GPS_SatsInView,13,10

                HRSOut "ALT: ", Dec GPS_Altitude, " Meters",13,10

                $ifdef _ReadGPVTG_
                HRSOut "Heading: ", Dec2 GPS_Heading, " Degrees",13,10
                HRSOut "Speed: ", Dec GPS_SpeedKm, " Km/h",13,10
                $endif

                ' LAT/LON Format D.Mm
                HRSOut "LAT_DMm: ", GPS_LatBearing, " ", Dec4 GPS_LAT_DMm,13,10
                HRSOut "LONG_DMm: ", GPS_LonBearing, " ", Dec4 GPS_LON_DMm,13,10
             
                ' LAT/LON Format D.M.S.
                HRSOut "LAT_DMS: ", Dec GPS_LAT_Degrees_dms, " Deg ", Dec2 GPS_LAT_Minutes_dms,39," "
                HRSOut Dec2 GPS_LAT_Seconds_dms,34," ", GPS_LatBearing,13,10
                HRSOut "LONG_DMS: ", Dec GPS_LON_Degrees_dms, " Deg ", Dec2 GPS_LON_Minutes_dms,39," "
                HRSOut Dec2 GPS_LON_Seconds_dms,34," ",GPS_LonBearing,13,10     

                ' LAT/LON Format D.d
                HRSOut "LAT_Dd: ", Dec5 GPS_LAT_Dd,13,10
                HRSOut "LONG_Dd: ", Dec5 GPS_LON_Dd,13,10   

            Case Else               ' Otherwise...
                HRSOut "Sats in view: ", Dec GPS_SatsInView,13,10, "No GPS Fix",13,10
                ' Datas used for calculations.
                GPS_LAT_Dd = 0
                GPS_LON_Dd = 0

        EndSelect       
        HRSOut 13,10

Calcdist:
        ' Display the distance/bearing between my QTH to Madrid on the serial terminal.
        $ifdef _CalcDistance_
        ' Distance => Dword variable, the decimals are deleted.
        ' AntBearing => Word variable, the decimals are deleted.
        'LATi_Dd = GPS_LAT_Dd
        'LONg_Dd = GPS_LON_Dd
        LATi_Dd = 41.47916
        LONg_Dd = 2.12500
        ' Variables:  Float ,  Float ,   Float ,  Float  ' Dword   ' Word
        CalcDistance(LATi_Dd, LONg_Dd, 40.39583, -3.70832, Distance, AntBearing)
        HRSOut "Distance: ", Dec Distance," Km",13,10
        HRSOut "Antenna Bearing: ", Dec AntBearing," Deg",13,10
        HRSOut 13,10
        $endif
        
        ' Display my true LAT/LON read from on line GPS and QTH Locator on the serial terminal.
'        LATi_Dd = 41.47916
'        LONg_Dd = 2.12500
'        GPS_LAT_Dd = 41.49420
'        GPS_LON_Dd = 2.13139       
        ' Variables:         Float   ,  Float    , String 
        ConvLatLonToQTHloc(GPS_LAT_Dd, GPS_LON_Dd, MyQTHStrg)
        HRSOut "LAT_Dd: ", Dec5 GPS_LAT_Dd,13,10
        HRSOut "LON_Dd: ", Dec5 GPS_LON_Dd,13,10
        HRSOut "My QTH Locator: ", MyQTHStrg,13,10
        HRSOut 13,10

'        LATi_Dd = -37.60416
'        LONg_Dd = 140.45834
'        ' Variables:        Float ,  Float , String 
'        ConvLatLonToQTHloc(LATi_Dd, LONg_Dd, MyQTHStrg)
'        HRSOut "LAT_Dd: ", Dec5 LATi_Dd,13,10
'        HRSOut "LON_Dd: ", Dec5 LONg_Dd,13,10
'        HRSOut "QTH Locator: ", MyQTHStrg,13,10
'        HRSOut 13,10

'        LATi_Dd = -12.10416
'        LONg_Dd = -77.04167
'        ' Variables:        Float ,  Float , String 
'        ConvLatLonToQTHloc(LATi_Dd, LONg_Dd, MyQTHStrg)
'        HRSOut "LAT_Dd: ", Dec5 LATi_Dd,13,10
'        HRSOut "LON_Dd: ", Dec5 LONg_Dd,13,10
'        HRSOut "QTH Locator: ", MyQTHStrg,13,10
'        HRSOut 13,10

        ' Variables:      String ,  Float , Float 
        ConvQTHtoLatLong(MyQTHStrg, LATi_Dd, LONg_Dd)
        HRSOut "My QTH Locator: ", MyQTHStrg,13,10
        HRSOut "LAT_Dd: ", Dec5 LATi_Dd,13,10
        HRSOut "LON_Dd: ", Dec5 LONg_Dd,13,10
        HRSOut 13,10

'        ' Variables:      String ,  Float , Float 
'        ConvQTHtoLatLong("QF02FJ", LATi_Dd, LONg_Dd)
'        HRSOut "QTH Locator: QF02FJ",13,10
'        HRSOut "LAT_Dd: ", Dec5 LATi_Dd,13,10
'        HRSOut "LON_Dd: ", Dec5 LONg_Dd,13,10
'        HRSOut 13,10

'        ' Variables:      String , Float  , Float 
'        ConvQTHtoLatLong("FH17LV", LATi_Dd, LONg_Dd)
'        HRSOut "QTH Locator: FH17LV",13,10
'        HRSOut "LAT_Dd: ", Dec5 LATi_Dd,13,10
'        HRSOut "LON_Dd: ", Dec5 LONg_Dd,13,10

    Wend    ' Do it forever

'===============================================================================    
    
    
