'****************************************************************
'*  Name    : GPS_WDG_ZX4120.inc                                *
'*  Project : Amicus18 GPS Receiver routines                    *
'*  Author  : [Les Johnson, 14th August 2011]                   *
'* Modified : by Alberto Freixanet 14 August 2017               *
'*  Notice  : Copyright (c) August 2011 [ ]                     *
'*          : All Rights Reserved                               *
'*  Date    : 14/08/2017                                        *
'*  Version : 1.0                                               *
'*  Notes   : Using the AMICUS18 Development Board              *
'*          : and Amicus18 GPA shield of Crownhill              *
'* Powered by PROTON DEVELOPMENT SUITE COMPILER V3.6.0.2        *
'****************************************************************
' Online NMEA XOR checksum calculator: http://www.hhhh.org/wiml/proj/nmeaxor.html
' To compile my bootloader, is better to use the PDS version 3.6.0.1 then the
' 3.5.9.9 that it could give some errors.
'
' 16/08/2017 update version 1.1
' Change the 2 first lines of calculations of the "Get Format D.d".
' Using Float variable instead of Dword variable.
'
' 17/08/2017 update version 1.2
' Checking the code with PDS version 3.6.0.2
' Using Heap directive.
' Modify some variables
'
' 18/08/2017 update version 1.3
' Clear the flags to print all files before reading them.
'
' 21/08/2017 update version 1.4
' - Change the order of checking in the Extract character routine: Case ",", 13, 0
' If the first chararcter to read is a comma, then return. It is faster.
' - Another mode to calculate LAT/LONG using the DM.m Float value from the GPS,
' Saving a lot of bytes than increasing the code speed.
' - Now it is possible to read 4 GSV files, because the GPS can send them.
' - Fixed error in the "DetectStartFile" code.
'
'###############################################################################
' This source code is free software and provided 'as is', any express or implied 
' warranties are disclaimed. you can redistribute it and/or modify.
' The author is not responsible of the use of this software.
' This program is distributed in the hope that it will be useful, but WITHOUT 
' ANY WARRANTY. However, a mention of the author and compiler would be appreciated.
'*******************************************************************************
' Este cdigo fuente es un software libre, puede redistribuirlo y/o modificarlo.
' El autor no se hace responsable del uso de este software.
' Este programa se distribuye con la esperanza de que sea til,
' pero SIN NINGUNA GARANTA. No obstante citar el autor y el compilador
' sera muy apreciado.
'*******************************************************************************
' Ce code source est un logiciel libre, vous pouvez le redistribuer et/ou modifier.
' L'auteur n'est pas responsable de l'utilisation de ce logiciel.
' Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE.
' Cependant mentionner l'auteur et le compileur serait trs aprci.
'
'###############################################################################
$ifndef _AMICUS18_GPS_
$define _AMICUS18_GPS_
'
' Amicus18 GPS Receiver routines
' Receive GPS data and parse it into date, time, and position
'
' Written by Les Johnson. 14th August 2011
' Modified by Alberto Freixanet, 14th August 2017 to increase GPS reading speed.
'
' Setup the software serial port to receive data from the GPS receiver
'    
    Declare Serial_Baud = 9600      ' GPS receiver baud rate
    Declare RsIn_Timeout = 1000     ' Set the Rsin timeout to 1 second (1000ms)  
    Declare RsIn_Pin = PORTA.0      ' From GPS receiver TX line
    Declare RsOut_Pin = PORTA.1     ' To GPS receiver RX line
    Declare RsIn_Mode = 0           ' Non-Inverted serial data
    Declare RsOut_Pace = 50         ' 50uS delay between Rsout characters.
'===============================================================================
'    Include "D:\MyIncFiles\DeclareLocalVars.inc"
    Include "INC\DeclareLocalVars.Inc"
'===============================================================================
'---Variables used in the GPS_StrToFloat_Sub subroutine-------------------------
    Dim PP_AARG As Float System
    Dim Atf_fPow10 As Float System
    Dim Atf_fResult As Float System
    Dim GPS_bChar As Byte System

    Dim GPS_FlagsSystem As Byte System
    Dim Atf_tSign As GPS_FlagsSystem.0
    Dim F_GetSign As GPS_FlagsSystem.1   

    Dim GPS_wFSR0 As FSR0L.Word
    Dim GPS_wFSR1 As FSR1L.Word
    Dim GPS_wFSR2 As FSR2L.Word
' 
'---Value extraction variables--------------------------------------------------

'---General purpose variables---------------------------------------------------
'    Dim GPS_LAT_Dd As Float
'    Dim GPS_LON_Dd As Float    

'---GPS variables--------------------------------------------------------------- 
    Dim GPS_eCharpos  As Byte
    Dim GPS_FixType As Byte             ' Hold the GPS fix type (0, 1, 2 or 3)
    Dim GPS_SatsInView As Byte          ' Holds the amount of satellites in view
    Dim GPS_Day As Byte                 ' Holds the Day value
    Dim GPS_TempDword As Dword
    Dim GPS_Month As Byte               ' Holds the Month value
    Dim GPS_Year As Byte                ' Holds the Year value
    Dim GPS_Hours As Byte               ' Holds the Hours value
    Dim GPS_Minutes As Byte             ' Holds the Minutes value
    Dim GPS_Seconds As Byte             ' Holds the Seconds value
    Dim GPS_LatBearing As Byte          ' Holds the Latitude qGPS_AntBearing character (N or S)
    Dim GPS_LonBearing As Byte          ' Holds the Longitude qGPS_AntBearing character (E or W)
    Dim GPS_LAT_DMm As Float            ' Holds the Latitude value
    Dim GPS_LON_DMm As Float            ' Holds the Longitude value
    Dim GPS_LAT_Dd As Float
    Dim GPS_LON_Dd As Float 
    Dim GPS_Altitude As Word            ' Holds the Altitude value
    Dim GPS_SpeedKm As Word             ' Holds the Speed value in Km/H
    Dim GPS_Heading As Float             ' Holds the Heading value in Degrees.
    Dim GPS_RMC_Valid As Byte
    Dim NumberOfmessages As Byte
    Dim MessageNumber As Byte
    Dim F_SelectNbRecord As Bit
    Dim ASCIIbyteCounter As Byte 
    
    Dim DO_Checksum As Byte
    Dim FinalLineChecksum As Byte

    ' GPS program flags
    Dim GPS_Flags As Byte
    Dim F_Do_CKS As GPS_Flags.0

    Dim GPS_FilesOKFlags As Byte
    Dim F_GSA_fileOK As GPS_FilesOKFlags.0
    Dim F_GSV1_fileOK As GPS_FilesOKFlags.1      
    Dim F_GSV2_fileOK As GPS_FilesOKFlags.2
    Dim F_GSV3_fileOK As GPS_FilesOKFlags.3
    Dim F_GSV4_fileOK As GPS_FilesOKFlags.4
    Dim F_RMC_fileOK As GPS_FilesOKFlags.5
    Dim F_VTG_fileOK As GPS_FilesOKFlags.6

    ' GPS error flags
    Dim GPS_FlagsError As Byte
    Dim F_ErrorGGAfile As GPS_FlagsError.0
    Dim F_ErrorGSAfile As GPS_FlagsError.1
    Dim F_ErrorGSV1file As GPS_FlagsError.2
    Dim F_ErrorGSV2file As GPS_FlagsError.3  
    Dim F_ErrorGSV3file As GPS_FlagsError.4
    Dim F_ErrorGSV4file As GPS_FlagsError.5    
    Dim F_ErrorRMCfile As GPS_FlagsError.6    
    Dim F_ErrorVTGfile As GPS_FlagsError.7

'---Checksums of the Headers of GPS lines----------------------------------------            
    Symbol cvGGA_Checksum = "G" ^ "P" ^ "G" ^ "G" ^ "A"
    Symbol cvGSA_Checksum = "G" ^ "P" ^ "G" ^ "S" ^ "A"
    Symbol cvRMC_Checksum = "G" ^ "P" ^ "R" ^ "M" ^ "C"
    Symbol cvVTG_Checksum = "G" ^ "P" ^ "V" ^ "T" ^ "G"
    Symbol cvGSV_Checksum = "G" ^ "P" ^ "G" ^ "S" ^ "V" 

    ' Sum of the Headers to decode the start of the files.
    Symbol cvGSA_Sum = 114
    Symbol cvGSV_Sum = 135
    Symbol cvRMC_Sum = 121
    Symbol cvVTG_Sum = 136

'----MAIDENHEAD-----------------------------------------------------------------
    Dim GPS_LAT_Degrees_dms As Byte
    Dim GPS_LAT_Minutes_dms As Byte
    Dim GPS_LAT_Seconds_dms As Float

    Dim GPS_LON_Degrees_dms As Byte
    Dim GPS_LON_Minutes_dms As Byte
    Dim GPS_LON_Seconds_dms As Float

'---GPS Arrays------------------------------------------------------------------
    Dim GPS_bDataString As String * 20 Heap     ' Temporary string
    Dim GPS_ggaRxBuffer[85] As Byte Heap        ' Holds the received characters from the GPS module
    $ifdef _ReadGPGSA_     
    Dim GPS_gsaRxBuffer[85] As Byte Heap        ' Holds the received characters from the GPS module
    $endif
    $ifdef _ReadGPGSV_
    Dim GPS_gsv1RxBuffer[85] As Byte Heap       ' Holds the received characters from the GPS module 
    Dim GPS_gsv2RxBuffer[85] As Byte Heap       ' Holds the received characters from the GPS module 
    Dim GPS_gsv3RxBuffer[85] As Byte Heap       ' Holds the received characters from the GPS module 
    Dim GPS_gsv4RxBuffer[85] As Byte Heap       ' Holds the received characters from the GPS module 
    $endif
    Dim GPS_rmcRxBuffer[85] As Byte Heap        ' Holds the received characters from the GPS module    
    $ifdef _ReadGPVTG_
    Dim GPS_vtgRxBuffer[85] As Byte Heap        ' Holds the received characters from the GPS module 
    $endif
'===============================================================================    
    GoTo _GPS_Main_Exit                             ' Jump over the GPS subroutines
'===============================================================================
' Convert an ASCII representation of a Floating Point value into a Floating Point variable
' Input     : GPS_bDataString holds the ASCII representation of the floating point value
' Output    : Atf_fResult holds the converted floating point value
' Notes     : Recognises -, +, and . as the beginning of the ASCII floating point value
'
GPS_StrToFloat_Sub:
    Atf_fPow10 = 1
    Atf_fResult = 0
    
    GPS_wFSR0 = VarPtr GPS_bDataString              ' Point FSR0L/H to the start of the string
    GPS_bChar = POSTINC0                            ' Get a character from the String
    Atf_tSign = 0
    If F_GetSign = 1 Then                               ' Do Get the sign is enabled?
        If GPS_bChar = "-" Then                         ' Have we found a "-" character?
            Atf_tSign = 1                               ' Indicate that the value is negative
            GPS_bChar = POSTINC0                        ' Yes. So Discard the "-" character
        Else If GPS_bChar = "+" Then                    ' Have we found a "+" character?
            GPS_bChar = POSTINC0                        ' Yes. So Discard the "+" character
        EndIf
    EndIf

    While 1 = 1                                     ' Scan the digits before the decimal point (if included)
        If GPS_bChar < "0" Then GoTo StrToFlt_Skp1  ' Exit the loop if non-numeric characters found
        If GPS_bChar > "9" Then GoTo StrToFlt_Skp1    

        Atf_fResult = Atf_fResult * 10              ' \
        PP_AARG = Atf_fResult + GPS_bChar           ' | Calculate the whole part of the floating point value
        Atf_fResult = PP_AARG - "0"                 ' /
        GPS_bChar = POSTINC0                        ' Get another character from the string
    Wend
StrToFlt_Skp1:
    If GPS_bChar = "." Then                         ' Have we found a "." character?
        GPS_bChar = POSTINC0                        ' Yes. So Discard the "." character

        While 1 = 1                                 ' Scan the digits after the decimal point (if included)
            If GPS_bChar < "0" Then GoTo StrToFlt_Skp2  ' Exit the loop if non-numeric characters found
            If GPS_bChar > "9" Then GoTo StrToFlt_Skp2            

            Atf_fPow10 = Atf_fPow10 * 10            ' \
            PP_AARG = GPS_bChar - "0"               '  | Calculate the fractional part of the floating point value
            PP_AARG = PP_AARG / Atf_fPow10          '  |
            Atf_fResult = Atf_fResult + PP_AARG     ' /
            GPS_bChar = POSTINC0                    ' Get another character from the string
        Wend
    EndIf
StrToFlt_Skp2:
    If Atf_tSign = 1 Then
        Atf_fResult = -Atf_fResult      ' Convert to negative if required
    EndIf
    Return   
'*******************************************************************************
'* Name    : ConvLatLonToQTHloc, ConvLatLonToQTHlocHRS                         *
'* Purpose : Convert D.d Format coordinates to Maidenhead grid                 *
'* Input   : LAT, LON                                                          *
'* Output  : QTH Locator = String                                              *
'* Notes   : Code from wiki, fixed & modified for macro by Alberto Freixanet   *
'*******************************************************************************
$ifdef _LatLonToLocator_
' Declare the local variables
    LocalFloat0(lat1F)
    LocalFloat1(lat2F)
    LocalFloat2(lon1F)
    LocalByte0(llon2)
    LocalByte1(llat3)    
    LocalByte2(llon3)
    LocalByte3(llat4)    
    LocalByte4(llon5)    
    LocalByte5(llat5)

' Declare some aliases
    Dim TempFloatLoc As lat1F
    Dim TempFloat1Loc As lat2F                

' Declare array for locator.
    Dim LocatorGrid As String * 6 ' Better in lower Bank

' Calculate the MakeMaidenhead value (QTH LOCATOR) from coordinates.
    $define ConvLatLonToQTHloc(pLAT,PLON,pString) '
    lat1F = pLAT                                  '
    lon1F = pLON                                  '
    GoSub ConvLatLonToQTHloc_Sub                  '
    pString = LocatorGrid

    $define ConvLatLonToQTHlocHRS(pLAT,pLON,pString) '
    lat1F = pLAT                                   '
    lon1F = pLON                                   '
    GoSub ConvLatLonToQTHloc_Sub                   '
    pString = LocatorGrid                          '
    HRSOut "GPS_LAT_Dd: ", Dec5 pLAT,13,10         '
    HRSOut "LONG_Dd: ", Dec5 pLON,13,10

ConvLatLonToQTHloc_Sub:
    ' Latitude.
    ' Using D.d Format
    lat1F = 90 +  lat1F         ' Input Latitude
    '---------------------------------------------------------------------------
    ' Second letter
    llat3 = Abs (lat1F / 10)      ' used for second letter
    ' Second digit
    llat4 = llat3 * 10
    llat4 = Abs (lat1F - llat4)    ' used for second digit
    ' Fourth letter
    lat2F = lat1F - (llat3 * 10)
    lat2F = lat2F - llat4
    llat5 = Abs (lat2F * 24)      ' used for forth letter
    '---------------------------------------------------------------------------
    ' Longitude
    ' Using D.d Format 
    lon1F = 180 + lon1F             ' Input Longitude
    '---------------------------------------------------------------------------
    ' first letter
    TempFloat1Loc = lon1F / 20
    llon2 = TempFloat1Loc                   ' first letter 
    ' first digit
    TempFloatLoc = lon1F - (llon2 * 20)         
    llon3 = Abs (TempFloatLoc / 2)          ' first digit
    ' third letter
    TempFloatLoc = TempFloatLoc - (llon3 * 2)
    llon5 = Abs (TempFloatLoc * 12)         ' third letter

    LocatorGrid#0 = llon2 + "A"     ' 1st character position 1
    LocatorGrid#1 = llat3 + "A"     ' 2nd character position 2
    LocatorGrid#2 = llon3 + "0"     ' 1st digit position 3
    LocatorGrid#3 = llat4 + "0"     ' 2nd digit position 4
    LocatorGrid#4 = llon5 + "A"     ' 3rd character position 5
    LocatorGrid#5 = llat5 + "A"     ' 4th character position 6
    LocatorGrid#6 = 0               ' Termination for string.

    Return
$endif
'*******************************************************************************
'* Name    : ConvQTHtoLatLong, ConvQTHtoLatLongHrS                             *
'* Purpose : Convert Locator grid To coordinates using D.d Format              *
'* Input   : QTH loc in String                                                 *
'* Output  : LAT, LON in D.d Format                                            *
'* Notes   : Code writen by John Drew, modified for macro by Alberto Freixanet *
'*******************************************************************************
$ifdef _LocatorToLatLon_
    LocalFloat0(LATloc_Dd)
    LocalFloat1(LONloc_Dd)    
    Dim InpQTHStrg As String * 6 ' Better in lower Bank

    ' Some aliases
    Dim gNum1 As InpQTHStrg#0
    Dim gNum2 As InpQTHStrg#1
    Dim gNum3 As InpQTHStrg#2
    Dim gNum4 As InpQTHStrg#3
    Dim gNum5 As InpQTHStrg#4
    Dim gNum6 As InpQTHStrg#5

' Definition variables:  String ,Float, Float
$define ConvQTHtoLatLong(pString, pLAT, PLON) '
    InpQTHStrg = pString                      '
    GoSub ConvertLocToCoordinates_Sub         '
    pLAT = LATloc_Dd                          '
    pLON = LONloc_Dd

' Definition variables:     String ,Float, Float
$define ConvQTHtoLatLongHRS(pString, pLAT, PLON) '
    InpQTHStrg = pString                     '
    GoSub ConvertLocToCoordinates_Sub        '
    pLAT = LATloc_Dd                         '
    pLON = LONloc_Dd                         '
    HRSOut "QTH Locator: ", InpQTHStrg,13,10 '                
    HRSOut "LAT: ", Dec6 LATloc_Dd,13,10     '
    HRSOut "LONG: ", Dec6 LONloc_Dd,13,10

ConvertLocToCoordinates_Sub:
    ' All characters must to be High Case.    
    gNum1 = InpQTHStrg#0 - 65
    gNum2 = InpQTHStrg#1 - 65
    gNum3 = InpQTHStrg#2 - 48
    gNum4 = InpQTHStrg#3 - 48
    gNum5 = InpQTHStrg#4 - 65
    gNum6 = InpQTHStrg#5 - 65
    
    LATloc_Dd = gNum2 * 10.0
    LATloc_Dd = LATloc_Dd + gNum4
    LATloc_Dd = LATloc_Dd + (gNum6 / 24.0)
    LATloc_Dd = LATloc_Dd - 89.979167
        
    LONloc_Dd = gNum1 * 20.0
    LONloc_Dd = LONloc_Dd + (gNum3 * 2)
    LONloc_Dd = LONloc_Dd + (gNum5 / 12)
    LONloc_Dd = LONloc_Dd - 179.95833    
    Return
$endif
(*
    QTH Locator: QF02FJ
    For testing purposes the result of the above are:
    Lat : -37.604167
    Long: 140.458343
My Code:
    Lat : -37.604167
    Long: 140.458340
    
    Delphi gives:
    Lat:-37.6041666
    Long:140.4583333
*) 
'*******************************************************************************
'* Name    : CalcDistance  (lat/Lon => D.d format)                             *
'* Purpose : Calculate the distance and bearing between two lat/Lon coodinates *
'* Input   : Home Lat(Float),Home Lon(Float),Remote Lat(Float),Remote Lon(Float)*
'* Output  : Distance(Dword), AntBearing(Word)                                 *
'* Notes   : The GPS_Distance calculation Code was written by John Drew.       *
'*******************************************************************************
$ifdef _CalcDistance_
'    'Define some constants
    Symbol PI = 3.14159265          'PI
    Symbol PIby2 = PI / 2           'pi dividend by 2
    Symbol TwoPI = PI * 2           'pi mulitplied by 2
    Symbol DegToRad = PI/180        'multiply by this to convert degrees to radians
    Symbol RadToDeg = 180/PI        'multiply by this to convert radians to degrees
    Symbol MinRadiusEarth = 6356    'polar radius        or semi minor axis
    Symbol MaxRadiusEarth = 6378    'equatorial radius   or semi major axis
    Symbol MinbyMax = 0.9965506     'ratio of min radius / max radius
    Symbol MeanRadius = 6367        '(Min + Max)/2
    Symbol Eccentricity = 0.9931131 'measure of the ellipsoid minradius squared/maxradius squared
    
    SMLocalFloat0(qHomeLong)     'set to home longitude in degrees -ve for West, +ve for East   (input)
    SMLocalFloat1(qHomeLat)      'set to home latitude in degrees -ve for South, +ve for North  (input)
    SMLocalFloat2(qRemoteLong)   'remote site long in degrees -ve for West, +ve for East        (input)
    SMLocalFloat3(qRemoteLat)    'remote site lat in degrees  -ve for South, +ve for South      (input
    SMLocalFloat4(qEccFactor)
    SMLocalFloat5(qAzimuth)
    SMLocalFloat6(qTemp1)        'qTemp1,2,3 used as a means of overcoming any problems with trig functions
    SMLocalFloat7(qTemp2)        'in complex lines
    SMLocalFloat8(qTemp3)
    SMLocalFloat9(qCO)    
    SMLocalFloat10(qSI)
    SMLocalFloat11(qN1)
    SMLocalFloat12(qN2)    
    SMLocalFloat13(qCA)
    SMLocalFloat14(qAL)    
    SMLocalFloat15(qEP)        
    SMLocalFloat16(qDist)
    SMLocalFloat17(qT1)    
    SMLocalFloat18(qT2)    
    SMLocalFloat19(qTH)          'used as a parameter for Atan2 routine
    SMLocalFloat20(qA2)    
    SMLocalFloat21(qA1)
    SMLocalFloat22(qXI)    
    SMLocalBit0(qArctan2Flag)
    Dim qGPS_AntBearing As qHomeLong ' calculated qGPS_AntBearing of the remote location from the home location (output)
    Dim qGPS_Distance As qHomeLat    ' qGPS_Distance between the two locations on a great circle          (output)
'-------------------------------------------------------------------------------
ArcTan2_Sub:
    If qArctan2Flag = 0 Then            'assign qSI and qCO while calculating central angle   
      qSI = qEccFactor * Sin(qTH)      'qTH is used to pass values to and from this routine
      qCO = Cos(qTH)
    EndIf
    
    qT1 = Abs(qSI)         'ArcTan2_Sub routine
    qT2 = Abs(qCO)         

    If qT1 <> 0 Or qT2 <> 0 Then
        If qT1 > qT2 Then 
            qTemp1 = ATan(qT2/qT1)
            qTH = PIby2 - qTemp1
        EndIf    
        If qT1 <= qT2 Then
            qTH = ATan(qT1/qT2)
        EndIf 
        If qCO < 0 Then
            qTH = PI - qTH
        EndIf
        If qSI < 0 Then
            qTH = -qTH
        EndIf 
    Else
        qTH = PIby2 
    EndIf       
    Return
'------------------------------------------------------------------------------- 
LongLine1_Sub:                                'just to separate out a long calculation from main code
    qTemp1 = DegToRad * (qHomeLong - qRemoteLong)
    qTemp1 = Cos(qTemp1)
    qTemp2 = Cos(qN1)
    qTemp3 = Cos(qN2)
    qTemp1 = qTemp1 * qTemp2 * qTemp3

    qTemp2 = Sin(qN1)
    qTemp3 = Sin(qN2)
    qTemp2 = qTemp2 * qTemp3
    
    qCO = qTemp1 + qTemp2
    qTemp1 = qCO * qCO
    qSI = Sqr(1 - qTemp1)
    Return
'-------------------------------------------------------------------------------     
LongLine2_Sub:                              'just to separate out a long calculation from main code
    qTemp1 = DegToRad * (qRemoteLong - qHomeLong)
    qTemp1 = Sin(qTemp1)
    qTemp2 = Cos(qN2)
    qTemp3 = Cos(qN1)
    qSI = qTemp1 * qTemp2 * qTemp3
   
    qTemp1 = Sin(qN1)
    qTemp2 = Cos(qCA)
    qTemp1 = qTemp1 * qTemp2
    qTemp2 = Sin(qN2)
    qCO = qTemp2 - qTemp1
    Return
'-------------------------------------------------------------------------------     
LongLine3_Sub:                               'just to separate out a long calculation from main code
    qTemp1 = 1 - (qSI * qSI)
    qTemp2 = Abs qTemp1
    qTemp3 = Sqr qTemp2
    qCO = qTemp3 * MinbyMax
    Return
'-------------------------------------------------------------------------------     
LongLine4_Sub:                               'just to separate out a long calculation from main code  
     qTemp1 = 1 - Eccentricity
     qTemp2 = Sin(qAL)
     qEP = qTemp2 * qTemp2 * qTemp1
     qTemp1 = Sqr(1-qEP)
     qEccFactor = 1 / qTemp1
     Return 
'-------------------------------------------------------------------------------    
GetDistance_Sub:  
    qTemp1 = qA2 - qA1
    qTemp2 = 1 - (qEP/4)          ' Here
    qTemp1 = qTemp1 * qTemp2
    qTemp2 = Sin (qA2 + qA2)    
    qTemp3 = Sin (qA1 + qA1)
    qTemp2 = qTemp2 - qTemp3
    qTemp2 = (qTemp2 * qEP) / 8   ' Here
    qDist = qTemp1 - qTemp2
    Return       
'-------------------------------------------------------------------------------  
'Initialise_Sub:
'    qHomeLat    = 51.59167    'home lat in decimal degrees
'    qHomeLong   = -1.246667   'home long in decimal degrees
'    qRemoteLat  = -50.00      'DX lat in decimal degrees
'    qRemoteLong = 89.38333    'DX long in decimal degrees
'    Return

' Madrid => Cerdanyola 437 Km.
'    qHomeLat    = 41.49404   'home lat in decimal degrees
'    qHomeLong   = 2.13154    'home long in decimal degrees
'    qRemoteLat  = 40.43777   'DX lat in decimal degrees
'    qRemoteLong = -3.27472   'DX long in decimal degrees
'    Return
'-------------------------------------------------------------------------------
        '            Float   , Float     , Float   , Float     ' Dword    ' Word
$define CalcDistance(pHomeLat, pHomeLon, pRemoteLat, pRemoteLon, pDistance, pAntBearing) '
    qHomeLat = pHomeLat            '
    qHomeLong = pHomeLon           '
    qRemoteLat = pRemoteLat        '
    qRemoteLong = pRemoteLon       '
    GoSub CalcDistance_Sub         '
    pDistance = qGPS_Distance      '
    pAntBearing = qGPS_AntBearing 
    
CalcDistance_Sub:
    qTH = DegToRad * qHomeLat
    qEccFactor = Eccentricity
    
    qArctan2Flag = 0         'get ArcTan2_Sub
    GoSub ArcTan2_Sub
    qN1 = qTH                 'adjusted latitude angle home
    
    qTH = DegToRad * qRemoteLat
    GoSub ArcTan2_Sub       'get atan2
    qN2 = qTH                 'adjusted angle remote
    GoSub LongLine1_Sub
    
    qArctan2Flag = 1
    GoSub ArcTan2_Sub
    qCA = qTH
    
    GoSub LongLine2_Sub
    GoSub ArcTan2_Sub
    qAzimuth = qTH               'qGPS_AntBearing of qDist object from home in radians
    
    If qAzimuth < 0 Then
        qAzimuth = qAzimuth + TwoPI
    EndIf
    If qN1 >= qN2 Then
        qT1 = qN1
        qN1 = qN2
        qN2 = qT1
    EndIf
    qTemp1 = Sin (qN1)
    qTemp2 = Sin (qCA)
    qSI = qTemp1 * qTemp2
    qTemp1 = Cos(qCA)
    qTemp2 = Sin(qN1)
    qTemp1 = qTemp1 * qTemp2
    qTemp2 = Sin(qN2)
    qCO = qTemp2 - qTemp1
               
    GoSub ArcTan2_Sub
    qXI = qTH
    qT1 = Sin(qXI)
    qT2 = Sin(qXI+qCA)

    If Abs(qT1) > Abs(qT2) Then 
        qSI = Sin(qN1) / qT1 
    Else 
        qSI = Sin(qN2) / qT2
    EndIf
    
    GoSub LongLine3_Sub
    
    GoSub ArcTan2_Sub
    qAL = qTH  
    
    GoSub LongLine4_Sub
    qTH = qXI
    qArctan2Flag = 0     

    GoSub ArcTan2_Sub
    qA1 = qTH
    
    qTH = qXI + qCA
    GoSub ArcTan2_Sub
    qA2 = qTH 
    
    GoSub GetDistance_Sub
    qGPS_Distance = MaxRadiusEarth * qDist    'qGPS_Distance - Float variable
    qGPS_AntBearing = qAzimuth * RadToDeg     'qGPS_AntBearing - convert to degrees - Float Variable

    Return        
$endif
'===============================================================================
' Extract a value based on the comma that precedes it
' Locate and extract the value following the comma number placed in GPS_bExtractItem
' Input     : GPS_bExtractItem holds the comma value to find the value after
'           : Array GPS_ggaRxBuffer holds the GPS text string
' Output    : The extracted text is returned in string GPS_bDataString
' Notes     : None
'
' Extract data from the GPS_ggaRxBuffer
    LocalByte0(GPS_bCharpos)
    LocalByte1(GPS_bCommaCount)
    LocalByte2(GPS_bLoop)    
    LocalByte3(GPS_bExtractItem)        

$define GPS_ExtractValueGGA(pCommaPosition) '
    WREG = pCommaPosition                   '
    GoSub __GPS_ExtractValueGGA_Sub
       
__GPS_ExtractValueGGA_Sub:
    GPS_bExtractItem = WREG
    Clear GPS_bDataString                               ' Clear the resulting string before we start
    GPS_bCommaCount = 0                                 ' Reset the comma counting variable
    GPS_bCharpos = 0                                    ' Start at the beginning of the array
    Repeat                                              ' Form a loop to scan the array
        WREG = GPS_ggaRxBuffer[GPS_bCharpos]            ' Scan the buffer array to parse
        If WREG = "," Then Inc GPS_bCommaCount          ' Increment GPS_bCommaCount if a comma is found

        If GPS_bCommaCount = GPS_bExtractItem Then      ' Have we found the correct comma?
            GPS_bLoop = 0                               ' Yes.  So....      
            Repeat                                      ' Create a loop
                Inc GPS_bCharpos                        ' Skip over the comma and keep scanning the array
                GPS_bChar = GPS_ggaRxBuffer[GPS_bCharpos] ' Extract the pieces of the value into GPS_bChar
                Select GPS_bChar                        ' Have we found a terminator character?
                     Case ",", 13, 0                    ' Yes. So...
                        Return                          ' Exit the subroutine
                EndSelect
                GPS_bDataString[GPS_bLoop] = GPS_bChar  ' Fill GPS_bDataString with the value
                Inc GPS_bLoop                           ' Point to the next data piece
            Until GPS_bCharpos >= SizeOf(GPS_ggaRxBuffer) ' Keep looping until a terminator is found, or the array runs out
            Return
        EndIf
        Inc GPS_bCharpos                                ' Move to the next byte in the array
    Until GPS_bCharpos >= SizeOf(GPS_ggaRxBuffer)       ' Loop until all the elements of the array are scanned
    Return 
'===============================================================================
' Extract a value based on the comma that precedes it
' Locate and extract the value following the comma number placed in GPS_cExtractItem
' Input     : GPS_cExtractItem holds the comma value to find the value after
'           : Array GPS_rmcRxBuffer holds the GPS text string
' Output    : The extracted text is returned in string GPS_bDataString
' Notes     : None
'
    LocalByte0(GPS_cCharpos)
    LocalByte1(GPS_cCommaCount)
    LocalByte2(GPS_cLoop)    
    LocalByte3(GPS_cExtractItem) 

$define GPS_ExtractValueRMC(pCommaPosition) '
    WREG = pCommaPosition                   '
    GoSub __GPS_ExtractValueRMC_Sub
       
__GPS_ExtractValueRMC_Sub:
    GPS_cExtractItem = WREG
    Clear GPS_bDataString                               ' Clear the resulting string before we start
    GPS_cCommaCount = 0                                 ' Reset the comma counting variable
    GPS_cCharpos = 0                                    ' Start at the beginning of the array
    Repeat                                              ' Form a loop to scan the array
        WREG = GPS_rmcRxBuffer[GPS_cCharpos]            ' Scan the buffer array to parse
        If WREG = "," Then Inc GPS_cCommaCount          ' Increment GPS_bCommaCount if a comma is found

        If GPS_cCommaCount = GPS_cExtractItem Then      ' Have we found the correct comma?
            GPS_cLoop = 0                               ' Yes.  So....      
            Repeat                                      ' Create a loop
                Inc GPS_cCharpos                        ' Skip over the comma and keep scanning the array
                GPS_bChar = GPS_rmcRxBuffer[GPS_cCharpos] ' Extract the pieces of the value into GPS_bChar
                Select GPS_bChar                        ' Have we found a terminator character?
                     Case ",", 13, 0                    ' Yes. So...
                        Return                          ' Exit the subroutine
                EndSelect
                GPS_bDataString[GPS_cLoop] = GPS_bChar  ' Fill GPS_bDataString with the value
                Inc GPS_cLoop                           ' Point to the next data piece
            Until GPS_cCharpos >= SizeOf(GPS_rmcRxBuffer) ' Keep looping until a terminator is found, or the array runs out
            Return
        EndIf
        Inc GPS_cCharpos                                ' Move to the next byte in the array
    Until GPS_cCharpos >= SizeOf(GPS_rmcRxBuffer)       ' Loop until all the elements of the array are scanned
    Return     
'===============================================================================
' Extract a value based on the comma that precedes it
' Locate and extract the value following the comma number placed in GPS_dExtractItem
' Input     : GPS_dExtractItem holds the comma value to find the value after
'           : Array GPS_vtgRxBuffer holds the GPS text string
' Output    : The extracted text is returned in string GPS_bDataString
' Notes     : None
'
$ifdef _ReadGPVTG_
    LocalByte0(GPS_dCharpos)
    LocalByte1(GPS_dCommaCount)
    LocalByte2(GPS_dLoop)    
    LocalByte3(GPS_dExtractItem) 

$define GPS_ExtractValueVTG(pCommaPosition) '
    WREG = pCommaPosition                   '
    GoSub __GPS_ExtractValueVTG_Sub
       
__GPS_ExtractValueVTG_Sub:
    GPS_dExtractItem = WREG
    Clear GPS_bDataString                               ' Clear the resulting string before we start
    GPS_dCommaCount = 0                                 ' Reset the comma counting variable
    GPS_dCharpos = 0                                    ' Start at the beginning of the array
    Repeat                                              ' Form a loop to scan the array
        WREG = GPS_vtgRxBuffer[GPS_dCharpos]            ' Scan the buffer array to parse
        If WREG = "," Then Inc GPS_dCommaCount          ' Increment GPS_bCommaCount if a comma is found

        If GPS_dCommaCount = GPS_dExtractItem Then      ' Have we found the correct comma?
            GPS_dLoop = 0                               ' Yes.  So....      
            Repeat                                      ' Create a loop
                Inc GPS_dCharpos                        ' Skip over the comma and keep scanning the array
                GPS_bChar = GPS_vtgRxBuffer[GPS_dCharpos] ' Extract the pieces of the value into GPS_bChar
                Select GPS_bChar                        ' Have we found a terminator character?
                     Case ",", 13, 0                    ' Yes. So...
                        Return                          ' Exit the subroutine
                EndSelect
                GPS_bDataString[GPS_dLoop] = GPS_bChar  ' Fill GPS_bDataString with the value
                Inc GPS_dLoop                           ' Point to the next data piece
            Until GPS_dCharpos >= SizeOf(GPS_vtgRxBuffer) ' Keep looping until a terminator is found, or the array runs out
            Return
        EndIf
        Inc GPS_dCharpos                                ' Move to the next byte in the array
    Until GPS_dCharpos >= SizeOf(GPS_vtgRxBuffer)       ' Loop until all the elements of the array are scanned
    Return     
$endif
'===============================================================================
' Extract a value based on the comma that precedes it
' Locate and extract the value following the comma number placed in GPS_fExtractItem
' Input     : GPS_fExtractItem holds the comma value to find the value after
'           : Array GPS_gsaRxBuffer holds the GPS text string
' Output    : The extracted text is returned in string GPS_bDataString
' Notes     : None
'
$ifdef _ReadGPGSA_
    LocalByte0(GPS_fCharpos)
    LocalByte1(GPS_fCommaCount)
    LocalByte2(GPS_fLoop)    
    LocalByte3(GPS_fExtractItem) 

$define GPS_ExtractValueGSA(pCommaPosition) '
    WREG = pCommaPosition                   '
    GoSub __GPS_ExtractValueGSA_Sub
       
__GPS_ExtractValueGSA_Sub:
    GPS_fExtractItem = WREG
    Clear GPS_bDataString                       ' Clear the resulting string before we start
    GPS_fCommaCount = 0                         ' Reset the comma counting variable
    GPS_fCharpos = 0                            ' Start at the beginning of the array
    Repeat                                      ' Form a loop to scan the array
        WREG = GPS_gsaRxBuffer[GPS_fCharpos]    ' Scan the buffer array to parse
        If WREG = "," Then Inc GPS_fCommaCount  ' Increment GPS_bCommaCount if a comma is found

        If GPS_fCommaCount = GPS_fExtractItem Then      ' Have we found the correct comma?
            GPS_fLoop = 0                               ' Yes.  So....      
            Repeat                                      ' Create a loop
                Inc GPS_fCharpos                        ' Skip over the comma and keep scanning the array
                GPS_bChar = GPS_gsaRxBuffer[GPS_fCharpos] ' Extract the pieces of the value into GPS_bChar
                Select GPS_bChar                        ' Have we found a terminator character?
                     Case ",", 13, 0                    ' Yes. So...
                        Return                          ' Exit the subroutine
                EndSelect
                GPS_bDataString[GPS_fLoop] = GPS_bChar    ' Fill GPS_bDataString with the value
                Inc GPS_fLoop                             ' Point to the next data piece
            Until GPS_fCharpos >= SizeOf(GPS_gsaRxBuffer) ' Keep looping until a terminator is found, or the array runs out
            Return
        EndIf
        Inc GPS_fCharpos                                ' Move to the next byte in the array
    Until GPS_fCharpos >= SizeOf(GPS_gsaRxBuffer)       ' Loop until all the elements of the array are scanned
    Return     
$endif
'===============================================================================
' Extract a value based on the comma that precedes it
' Locate and extract the value following the comma number placed in GPS_gExtractItem
' Input     : GPS_gExtractItem holds the comma value to find the value after
'           : Array GPS_gsv1RxBuffer holds the GPS text string
' Output    : The extracted text is returned in string GPS_bDataString
' Notes     : None
'
$ifdef _ReadGPGSV_
    LocalByte0(GPS_gCharpos)
    LocalByte1(GPS_gCommaCount)
    LocalByte2(GPS_gLoop)    
    LocalByte3(GPS_gExtractItem) 

$define GPS_ExtractValueGSV1(pCommaPosition) '
    WREG = pCommaPosition                    '
    GoSub __GPS_ExtractValueGSV1_Sub
       
__GPS_ExtractValueGSV1_Sub:
    GPS_gExtractItem = WREG
    Clear GPS_bDataString                       ' Clear the resulting string before we start
    GPS_gCommaCount = 0                         ' Reset the comma counting variable
    GPS_gCharpos = 0                            ' Start at the beginning of the array
    Repeat                                      ' Form a loop to scan the array
        WREG = GPS_gsv1RxBuffer[GPS_gCharpos]   ' Scan the buffer array to parse
        If WREG = "," Then Inc GPS_gCommaCount  ' Increment GPS_bCommaCount if a comma is found

        If GPS_gCommaCount = GPS_gExtractItem Then      ' Have we found the correct comma?
            GPS_gLoop = 0                               ' Yes.  So....      
            Repeat                                      ' Create a loop
                Inc GPS_gCharpos                        ' Skip over the comma and keep scanning the array
                GPS_bChar = GPS_gsv1RxBuffer[GPS_gCharpos] ' Extract the pieces of the value into GPS_bChar
                Select GPS_bChar                        ' Have we found a terminator character?
                     Case ",", 13, 0                    ' Yes. So...
                        Return                          ' Exit the subroutine
                EndSelect
                GPS_bDataString[GPS_gLoop] = GPS_bChar     ' Fill GPS_bDataString with the value
                Inc GPS_gLoop                              ' Point to the next data piece
            Until GPS_gCharpos >= SizeOf(GPS_gsv1RxBuffer) ' Keep looping until a terminator is found, or the array runs out
            Return
        EndIf
        Inc GPS_gCharpos                                ' Move to the next byte in the array
    Until GPS_gCharpos >= SizeOf(GPS_gsv1RxBuffer)      ' Loop until all the elements of the array are scanned
    Return     
$endif
'===============================================================================
' Extract a value based on the comma that precedes it
' Locate and extract the value following the comma number placed in GPS_hExtractItem
' Input     : GPS_hExtractItem holds the comma value to find the value after
'           : Array GPS_gsv2RxBuffer holds the GPS text string
' Output    : The extracted text is returned in string GPS_bDataString
' Notes     : None
'
$ifdef _ReadGPGSV_
    LocalByte0(GPS_hCharpos)
    LocalByte1(GPS_hCommaCount)
    LocalByte2(GPS_hLoop)    
    LocalByte3(GPS_hExtractItem) 

$define GPS_ExtractValueGSV2(pCommaPosition) '
    WREG = pCommaPosition                    '
    GoSub __GPS_ExtractValueGSV2_Sub
       
__GPS_ExtractValueGSV2_Sub:
    GPS_hExtractItem = WREG
    Clear GPS_bDataString                       ' Clear the resulting string before we start
    GPS_hCommaCount = 0                         ' Reset the comma counting variable
    GPS_hCharpos = 0                            ' Start at the beginning of the array
    Repeat                                      ' Form a loop to scan the array
        WREG = GPS_gsv2RxBuffer[GPS_hCharpos]   ' Scan the buffer array to parse
        If WREG = "," Then Inc GPS_hCommaCount  ' Increment GPS_bCommaCount if a comma is found

        If GPS_hCommaCount = GPS_hExtractItem Then      ' Have we found the correct comma?
            GPS_hLoop = 0                               ' Yes.  So....      
            Repeat                                      ' Create a loop
                Inc GPS_hCharpos                        ' Skip over the comma and keep scanning the array
                GPS_bChar = GPS_gsv2RxBuffer[GPS_hCharpos] ' Extract the pieces of the value into GPS_bChar
                Select GPS_bChar                        ' Have we found a terminator character?
                     Case ",", 13, 0                    ' Yes. So...
                        Return                          ' Exit the subroutine
                EndSelect
                GPS_bDataString[GPS_hLoop] = GPS_bChar     ' Fill GPS_bDataString with the value
                Inc GPS_hLoop                              ' Point to the next data piece
            Until GPS_hCharpos >= SizeOf(GPS_gsv2RxBuffer) ' Keep looping until a terminator is found, or the array runs out
            Return
        EndIf
        Inc GPS_hCharpos                                ' Move to the next byte in the array
    Until GPS_hCharpos >= SizeOf(GPS_gsv2RxBuffer)      ' Loop until all the elements of the array are scanned
    Return     
$endif
'===============================================================================
' Extract a value based on the comma that precedes it
' Locate and extract the value following the comma number placed in GPS_iExtractItem
' Input     : GPS_iExtractItem holds the comma value to find the value after
'           : Array GPS_gsv3RxBuffer holds the GPS text string
' Output    : The extracted text is returned in string GPS_bDataString
' Notes     : None
'
$ifdef _ReadGPGSV_
    LocalByte0(GPS_iCharpos)
    LocalByte1(GPS_iCommaCount)
    LocalByte2(GPS_iLoop)    
    LocalByte3(GPS_iExtractItem) 

$define GPS_ExtractValueGSV3(pCommaPosition) '
    WREG = pCommaPosition                    '
    GoSub __GPS_ExtractValueGSV3_Sub
       
__GPS_ExtractValueGSV3_Sub:
    GPS_iExtractItem = WREG
    Clear GPS_bDataString                       ' Clear the resulting string before we start
    GPS_iCommaCount = 0                         ' Reset the comma counting variable
    GPS_iCharpos = 0                            ' Start at the beginning of the array
    Repeat                                      ' Form a loop to scan the array
        WREG = GPS_gsv3RxBuffer[GPS_iCharpos]   ' Scan the buffer array to parse
        If WREG = "," Then Inc GPS_iCommaCount  ' Increment GPS_bCommaCount if a comma is found

        If GPS_iCommaCount = GPS_iExtractItem Then      ' Have we found the correct comma?
            GPS_iLoop = 0                               ' Yes.  So....      
            Repeat                                      ' Create a loop
                Inc GPS_iCharpos                        ' Skip over the comma and keep scanning the array
                GPS_bChar = GPS_gsv3RxBuffer[GPS_iCharpos] ' Extract the pieces of the value into GPS_bChar
                Select GPS_bChar                        ' Have we found a terminator character?
                     Case ",", 13, 0                    ' Yes. So...
                        Return                          ' Exit the subroutine
                EndSelect
                GPS_bDataString[GPS_iLoop] = GPS_bChar     ' Fill GPS_bDataString with the value
                Inc GPS_iLoop                              ' Point to the next data piece
            Until GPS_iCharpos >= SizeOf(GPS_gsv3RxBuffer) ' Keep looping until a terminator is found, or the array runs out
            Return
        EndIf
        Inc GPS_iCharpos                                ' Move to the next byte in the array
    Until GPS_iCharpos >= SizeOf(GPS_gsv3RxBuffer)      ' Loop until all the elements of the array are scanned
    Return     
$endif
'===============================================================================
' Extract a value based on the comma that precedes it
' Locate and extract the value following the comma number placed in GPS_jExtractItem
' Input     : GPS_jExtractItem holds the comma value to find the value after
'           : Array GPS_gsv4RxBuffer holds the GPS text string
' Output    : The extracted text is returned in string GPS_bDataString
' Notes     : None
'
$ifdef _ReadGPGSV_
    LocalByte0(GPS_jCharpos)
    LocalByte1(GPS_jCommaCount)
    LocalByte2(GPS_jLoop)    
    LocalByte3(GPS_jExtractItem) 

$define GPS_ExtractValueGSV4(pCommaPosition) '
    WREG = pCommaPosition                    '
    GoSub __GPS_ExtractValueGSV4_Sub
       
__GPS_ExtractValueGSV4_Sub:
    GPS_jExtractItem = WREG
    Clear GPS_bDataString                       ' Clear the resulting string before we start
    GPS_jCommaCount = 0                         ' Reset the comma counting variable
    GPS_jCharpos = 0                            ' Start at the beginning of the array
    Repeat                                      ' Form a loop to scan the array
        WREG = GPS_gsv4RxBuffer[GPS_jCharpos]   ' Scan the buffer array to parse
        If WREG = "," Then Inc GPS_jCommaCount  ' Increment GPS_bCommaCount if a comma is found

        If GPS_jCommaCount = GPS_jExtractItem Then      ' Have we found the correct comma?
            GPS_jLoop = 0                               ' Yes.  So....      
            Repeat                                      ' Create a loop
                Inc GPS_jCharpos                        ' Skip over the comma and keep scanning the array
                GPS_bChar = GPS_gsv4RxBuffer[GPS_jCharpos] ' Extract the pieces of the value into GPS_bChar
                Select GPS_bChar                        ' Have we found a terminator character?
                     Case ",", 13, 0                    ' Yes. So...
                        Return                          ' Exit the subroutine
                EndSelect
                GPS_bDataString[GPS_jLoop] = GPS_bChar     ' Fill GPS_bDataString with the value
                Inc GPS_jLoop                              ' Point to the next data piece
            Until GPS_jCharpos >= SizeOf(GPS_gsv4RxBuffer) ' Keep looping until a terminator is found, or the array runs out
            Return
        EndIf
        Inc GPS_jCharpos                                ' Move to the next byte in the array
    Until GPS_jCharpos >= SizeOf(GPS_gsv4RxBuffer)      ' Loop until all the elements of the array are scanned
    Return     
$endif
'===============================================================================
' Convert the byte elements of string GPS_bDataString to integer values
' Input     : GPS_bDataString holds the ASCII value
' Output    : GPS_bDataString holds the integer values of the characters
' Notes     : GPS_bChar is a global variable
'
_GPS_ElementsToInt_Sub:
    GPS_wFSR0 = VarPtr GPS_bDataString  ' Point FSR0L/H to the start of the string
    While 1 = 1                         ' Create a loop
        GPS_bChar = INDF0               ' Get a byte from the string
        If GPS_bChar = 0 Then Return    ' Exit the subroutine when the string reaches the end
        POSTINC0 = GPS_bChar - 48       ' Load GPS_bDataString with the integer equivalent of the character 
    Wend
    Return 
'===============================================================================
' Extract the the 2 last bytes of the file for the final checksum.
' Input     : final checksum ASCII value of the file
' Output    : Line checksum: FinalLineChecksum
' Notes     : none
'
MakeFinalChecksum_Sub:
    GPS_wFSR0 = GPS_wFSR0 - 3       ' Point on the first ASCII hex byte checksum. 
    GPS_bChar = POSTINC0            ' Reading the next buffer byte.
    ' Transform the ASCII Hex to Bin HEX  from GPS_bChar and return the value in WREG.
    If GPS_bChar > 64 Then      
        WREG = GPS_bChar - 55   ' Get (A to F)
    Else
        WREG = GPS_bChar - 48   ' Get (0 to 9)
    EndIf
    FinalLineChecksum = WREG << 4   ' Loading the first checksum HEX byte.
    GPS_bChar = INDF0               ' Reading the last buffer byte.
    ' Transform the ASCII Hex to Bin HEX  from GPS_bChar and return the value in WREG.
    If GPS_bChar > 64 Then      
        WREG = GPS_bChar - 55   ' Get (A to F)
    Else
        WREG = GPS_bChar - 48   ' Get (0 to 9)
    EndIf
    FinalLineChecksum = FinalLineChecksum | WREG  ' Loading the second checksum HEX byte.
    Return
'===============================================================================
' Parse GPS data
' Extract the location, time and date information from the GPS module
' Input     : None
' Output    : GPS_Day Holds the Day value
'           : GPS_Month Holds the Month value
'           : GPS_Year Holds the Year value
'           : GPS_Hours Holds the Hours value
'           : GPS_Minutes Holds the Minutes value
'           : GPS_Seconds Holds the Seconds value
'           : GPS_LAT_DMm Holds the floating point Latitude value
'           : GPS_LatBearing Holds the Latitude qGPS_AntBearing character (N or S)
'           : GPS_LON_DMm Holds the floating point Longitude value
'           : GPS_LonBearing Holds the Longitude qGPS_AntBearing character (E or W)
'           : GPS_FixType Hold the GPS fix type (0, 1, 2 or 3)
'           : GPS_SatsInView Holds the amount of satellites in view
' Notes     : Writing the RXBuffers first increases speed.
' 
$define GPS_Parse() GoSub GPS_Parse_Sub

GPS_Parse_Sub:
    GPS_FlagsError = 0      ' Clear all error flags.   
    GPS_Flags = 0           ' Clear all general flags.
'-------------------------------------------------------------------------------
' Refresh all the lines of NMEA text for the terminal.
' Input     : All buffers, 
' Output    : GPS_FilesOKFlags
' Notes     : If Enable this line to see the received files only, every second.
'             If this line is disabled all files are refreshed every second or every 3 seconds.
'
'    GPS_FilesOKFlags = 0
'-------------------------------------------------------------------------------
' Receive the lines of NMEA text from the GPS module
' Input     : Received serial data from the GPS module
' Output    : GPS_ggaRxBuffer holds the line of NMEA text
' Notes     : Very fast routines at FOSC = 80Mhz!!!!
'           : The Checksum control is included within the reading routine.
'-------------------------------------------------------------------------------
' $GPGGA file - Global Positioning System Fix Data
    F_Do_CKS = 0
    DO_Checksum = cvGGA_Checksum                ' Load checksum GPGGA word.
    GPS_eCharpos = 0
    GPS_wFSR0 = VarPtr GPS_ggaRxBuffer          ' Point FSR0L/H to the start of the buffer

    RsIn {__GPS_GGA_Timeout},Wait("$GPGGA")      ' Wait for the $GPGGA string (with a timeout)
    ' Receive the rest of the NMEA text into GPS_ggaRxBuffer
    Repeat                                      ' Create a loop to receive the serial string
        RsIn {__GPS_GGA_Timeout},WREG            ' Receive a character serially from the GPS module
        POSTINC0 = WREG                         ' Load a value of array GPS_ggaRxBuffer        
        GPS_bChar = WREG
        ' Make checksum of all of bytes of the file until "*" character.
        If F_Do_CKS = 0 Then
            If GPS_bChar <> "*" Then             
                DO_Checksum = DO_Checksum ^ GPS_bChar ' Do checksum
            Else
                F_Do_CKS = 1                    ' Stop DO checksum.
            EndIf
        EndIf
        If GPS_bChar = 13 Then GoTo SuccesReadingGGA ' Exit the loop when the string reaches the end
        Inc GPS_eCharpos
    Until GPS_eCharpos = SizeOf(GPS_ggaRxBuffer)  ' Repeat the loop until the buffer runs out (security)
    '---------------------------------------------------------------------------
    F_ErrorGGAfile = 1                            ' Get an error, the Carriage Return not found.
    GoTo ReadGGAexit                              ' If the Frame is bad GoTo error.

SuccesReadingGGA:
    ' Check the final checksum of file.
    GoSub MakeFinalChecksum_Sub
    If DO_Checksum <> FinalLineChecksum Then     ' Compare the 2 checksums.
        F_ErrorGGAfile = 1                         ' Get an error, checksum fails.
    EndIf

ReadGGAexit:
    If F_ErrorGGAfile = 1 Then
        HRSOut "$GPGGA fails",13,10
        Return  ' Return of GPS_Parse() routine.
    EndIf
'###############################################################################
' Read and detect the file after the first line $GPGGA, because the following lines
' change every second.
'===============================================================================
' Read And detect the file after the first Line $GPGGA, because the following lines
' change every second.
' Input     : RsIn {__GPS_StartTimeout},Wait("$")
' Output    : goto the destination file reading
' Notes     : None
'
$ifdef _SynchronizedOperation_
DetectStartFile:
    GPS_bChar = 0
    ASCIIbyteCounter = 5
    RsIn {__GPS_StartTimeout},Wait("$") ' Wait for the $ start character of file. (with a timeout)
    ' Receive the rest of the HEADER.
    Repeat                              ' Create a loop to receive the serial string
        Dec ASCIIbyteCounter
        RsIn {__GPS_StartTimeout},WREG  ' Receive a character serially from the GPS module
        GPS_bChar = GPS_bChar + WREG
    Until ASCIIbyteCounter = 0          ' Exit the loop when the string reaches the end

    Select GPS_bChar
        $ifdef _ReadGPGSA_
        Case cvGSA_Sum          ' Symbol cvGSA_Sum = 114
            GoTo ReadGSA_Sub    ' Read the "$GPGSA" file.
        $endif
        $ifdef _ReadGPGSV_
        Case cvGSV_Sum          ' Symbol cvGSV_Sum = 135
            GoTo ReadGSV_Sub    ' Read the 1, 2, 3 or 4 "$GPGSV" files.        
        $endif
        Case cvRMC_Sum          ' Symbol cvRMC_Sum = 121
            GoTo ReadRMC_Sub    ' Read the "$GPRMC" file.    
        $ifdef _ReadGPVTG_
        Case cvVTG_Sum          ' Symbol cvVTG_Sum = 136
            GoTo ReadVTG_Sub    ' Read the "$GPVTG" file. Last one & print all files.
        $endif
    EndSelect
    GoTo DetectStartFile        ' Noother file detected. 
$endif
'###############################################################################
' Loading the "$GPGSA" file - GPS DOP and active satellites
' Input     : RsIn {__GPS_GSA_Timeout},WREG
' Output    : GPS_gsaRxBuffer
' Notes     : None
'
$ifdef _ReadGPGSA_
ReadGSA_Sub:
    F_GSA_fileOK = 0                            '' Initialize to print the GSA file.
    F_Do_CKS = 0
    DO_Checksum = cvGSA_Checksum                ' Load checksum GPGGA word.
    GPS_eCharpos = 0
    GPS_wFSR0 = VarPtr GPS_gsaRxBuffer          ' Point FSR0L/H to the start of the buffer

    $ifndef _SynchronizedOperation_
    RsIn {__GPS_GSA_Timeout},Wait("$GPGSA")      ' Wait for the $GPGGA string (with a timeout)
    $endif
    ' Receive the rest of the NMEA text into GPS_gsaRxBuffer
    Repeat                                      ' Create a loop to receive the serial string
        RsIn {__GPS_GSA_Timeout},WREG            ' Receive a character serially from the GPS module
        POSTINC0 = WREG                         ' Load a value of array GPS_ggaRxBuffer        
        GPS_bChar = WREG
        ' Make checksum of all of bytes of the file until "*" character.
        If F_Do_CKS = 0 Then
            If GPS_bChar <> "*" Then             
                DO_Checksum = DO_Checksum ^ GPS_bChar ' Do checksum
            Else
                F_Do_CKS = 1                    ' Stop DO checksum.
            EndIf
        EndIf
        If GPS_bChar = 13 Then GoTo SuccesReadingGSA ' Exit the loop when the string reaches the end
        Inc GPS_eCharpos
    Until GPS_eCharpos = SizeOf(GPS_gsaRxBuffer)  ' Repeat the loop until the buffer runs out (security)
    '---------------------------------------------------------------------------
    F_ErrorGSAfile = 1                            ' Get an error, the Carriage Return not found.
    GoTo ReadGSAexit                              ' If the Frame is bad goto error.

SuccesReadingGSA:
    ' Check the final checksum of file.
    GoSub MakeFinalChecksum_Sub
    If DO_Checksum <> FinalLineChecksum Then      ' Compare the 2 checksums.
        F_ErrorGSAfile = 1                        ' Get an error, checksum fails.
    EndIf

ReadGSAexit:
    If F_ErrorGSAfile = 1 Then
        HRSOut "$GPGSA fails",13,10
    Else
        F_GSA_fileOK = 1
    EndIf
    $ifdef _SynchronizedOperation_
    GoTo DetectStartFile
    $endif
$endif
'*******************************************************************************
' This message transfers information about satellites in view.
' Each record contains the information for up to 4 channels, allowing up to 12
' satellites in view. In the final record of the sequence the unused channel fields
' are left blank with commas to indicate that a field has been omitted.
' Input     : RsIn {__GPS_GSV_Timeout},Wait("$GPGSV") 
' Output    : GPS_gsv1RxBuffer
' Notes     : None
'
$ifdef _ReadGPGSV_
' Loading the "$GPGSV" files (1 to 3) - GPS Satellites in view
    '---Read First GSV message-------------------------------------------------- 
ReadGSV_Sub:
    F_GSV1_fileOK = 0                       ' Initialize to print the GSV1 file.
    F_Do_CKS = 0
    F_SelectNbRecord = 0
    ASCIIbyteCounter = 0
    DO_Checksum = cvGSV_Checksum            ' Load checksum GPGSV word.
    GPS_eCharpos = 0
    GPS_wFSR0 = VarPtr GPS_gsv1RxBuffer     ' Point FSR0L/H to the start of the buffer

    $ifndef _SynchronizedOperation_
    RsIn {__GPS_GSV_Timeout},Wait("$GPGSV")     ' Wait for the $GPGSV string (with a timeout)
    $endif
    ' Receive the rest of the NMEA text into GPS_gsv1RxBuffer
    Repeat                                      ' Create a loop to receive the serial string
        RsIn {__GPS_GSV_Timeout},WREG            ' Receive a character serially from the GPS module
        POSTINC0 = WREG                         ' Load a value of array GPS_gsv1RxBuffer        
        GPS_bChar = WREG
        If F_SelectNbRecord = 0 Then            ' Count the bytes...
            Inc ASCIIbyteCounter
            If ASCIIbyteCounter = 2 Then            ' Did NumberOfmessages detected?
                NumberOfmessages = GPS_bChar - 48   ' Convert ASCII to Bin
                F_SelectNbRecord = 1            
            EndIf
            ' Same code as above.
'            Select ASCIIbyteCounter
'                Case 2
'                    NumberOfmessages = GPS_bChar - 48   ' Convert ASCII to Bin
'                    F_SelectNbRecord = 1
''                Case 3
''                    MessageNumber = GPS_bChar - 48      ' Convert ASCII to Bin        
''                    F_SelectNbRecord = 1
'            EndSelect
        EndIf
        ' Make checksum of all of bytes of the file until "*" character.
        If F_Do_CKS = 0 Then
            If GPS_bChar <> "*" Then             
                DO_Checksum = DO_Checksum ^ GPS_bChar ' Do checksum
            Else
                F_Do_CKS = 1                    ' Stop DO checksum.
            EndIf
        EndIf
        If GPS_bChar = 13 Then GoTo SuccesReadingGSV1   ' Exit the loop when the string reaches the end
        Inc GPS_eCharpos
    Until GPS_eCharpos = SizeOf(GPS_gsv1RxBuffer)   ' Repeat the loop until the buffer runs out
    '---------------------------------------------------------------------------
    F_ErrorGSV1file = 1                             ' Get an error, the Carriage Return not found.
    GoTo ReadGSV1exit

SuccesReadingGSV1:
    If NumberOfmessages > 4 Then
        F_ErrorGSV1file = 1                     ' If the Frame is bad GoTo error.
        GoTo ReadGSV1exit
    EndIf
    ' Check the final checksum of file.
    GoSub MakeFinalChecksum_Sub
    If DO_Checksum <> FinalLineChecksum Then     ' Compare the 2 checksums.
        F_ErrorVTGfile = 1
    EndIf

ReadGSV1exit:
    If F_ErrorGSV1file = 1 Then
        HRSOut "$GPGSV-1 fails",13,10
        $ifdef _SynchronizedOperation_
        GoTo DetectStartFile
        $endif
    Else
        F_GSV1_fileOK = 1
    EndIf
    '---Load Second GSV message------------------------------------------------- 
    ' This message transfers information about satellites in view.
    ' Each record contains the information for up to 4 channels, allowing up to 12
    ' satellites in view. In the final record of the sequence the unused channel fields
    ' are left blank with commas to indicate that a field has been omitted.
    ' $GPGSV
    ' Input     : RsIn {__GPS_GSV_Timeout},Wait("$GPGSV") 
    ' Output    : GPS_gsv2RxBuffer
    ' Notes     : None
    '
    If NumberOfmessages > 1 Then
        F_GSV2_fileOK = 0                           ' Initialize to print the GSV2 file.
        F_Do_CKS = 0
        DO_Checksum = cvGSV_Checksum               ' Load checksum GPGSV word.
'        F_SelectNbRecord = 0
'        ASCIIbyteCounter = 0
        GPS_eCharpos = 0
        GPS_wFSR0 = VarPtr GPS_gsv2RxBuffer ' Point FSR0L/H to the start of the buffer

        RsIn {__GPS_GSV_Timeout},Wait("$GPGSV")      ' Wait for the $GPGSV string (with a timeout)
        ' Receive the rest of the NMEA text into GPS_gsv2RxBuffer
        Repeat                                      ' Create a loop to receive the serial string
            RsIn {__GPS_GSV_Timeout},WREG            ' Receive a character serially from the GPS module
            POSTINC0 = WREG                         ' Load a value of array GPS_gsv2RxBuffer        
            GPS_bChar = WREG
'            If F_SelectNbRecord = 0 Then
'                Inc ASCIIbyteCounter
'                If ASCIIbyteCounter = 3 Then
'                    MessageNumber = GPS_bChar - 48  ' Convert ASCII to Bin        
'                    F_SelectNbRecord = 1
'                EndIf
'            EndIf
            ' Make checksum of all of bytes of the file until "*" character.
            If F_Do_CKS = 0 Then
                If GPS_bChar <> "*" Then             
                    DO_Checksum = DO_Checksum ^ GPS_bChar ' Do checksum
                Else
                    F_Do_CKS = 1                    ' Stop DO checksum.
                EndIf
            EndIf
            If GPS_bChar = 13 Then GoTo SuccesReadingGSV2   ' Exit the loop when the string reaches the end
            Inc GPS_eCharpos
        Until GPS_eCharpos = SizeOf(GPS_gsv2RxBuffer) ' Repeat the loop until the buffer runs out
        '-----------------------------------------------------------------------
        F_ErrorGSV2file = 1                           ' Get an error, the Carriage Return not found.
        GoTo ReadGSV2exit                             ' GoTo error.

SuccesReadingGSV2:
        ' Check the final checksum of file.
        GoSub MakeFinalChecksum_Sub
        If DO_Checksum <> FinalLineChecksum Then     ' Compare the 2 checksums.
            F_ErrorGSV2file = 1
        EndIf
ReadGSV2exit:
        If F_ErrorGSV2file = 1 Then
            HRSOut "$GPGSV-2 fails",13,10
            $ifdef _SynchronizedOperation_
            GoTo DetectStartFile
            $endif
        Else
            F_GSV2_fileOK = 1
        EndIf
    EndIf
    '---Load Third GSV message-------------------------------------------------- 
    ' This message transfers information about satellites in view.
    ' Each record contains the information for up to 4 channels, allowing up to 12
    ' satellites in view. In the final record of the sequence the unused channel fields
    ' are left blank with commas to indicate that a field has been omitted.
    ' $GPGSV
    ' Input     : RsIn {__GPS_GSV_Timeout},Wait("$GPGSV") 
    ' Output    : GPS_gsv3RxBuffer
    ' Notes     : None
    '
    If NumberOfmessages > 2 Then
        F_GSV3_fileOK = 0                       ' Initialize to print the GSV3 file.
        F_Do_CKS = 0
        GPS_eCharpos = 0
        DO_Checksum = cvGSV_Checksum           ' Load checksum GPGSV word.
        GPS_wFSR0 = VarPtr GPS_gsv3RxBuffer     ' Point FSR0L/H to the start of the buffer

        RsIn {__GPS_GSV_Timeout},Wait("$GPGSV")      ' Wait for the $GPGSV string (with a timeout)
        ' Receive the rest of the NMEA text into GPS_gsv3RxBuffer
        Repeat                                      ' Create a loop to receive the serial string
            RsIn {__GPS_GSV_Timeout},WREG            ' Receive a character serially from the GPS module
            POSTINC0 = WREG                         ' Load a value of array GPS_gsv3RxBuffer        
            GPS_bChar = WREG
            ' Make checksum of all of bytes of the file until "*" character.
            If F_Do_CKS = 0 Then
                If GPS_bChar <> "*" Then             
                    DO_Checksum = DO_Checksum ^ GPS_bChar ' Do checksum
                Else
                    F_Do_CKS = 1                    ' Stop DO checksum.
                EndIf
            EndIf
            If GPS_bChar = 13 Then GoTo SuccesReadingGSV3  ' Exit the loop when the string reaches the end
            Inc GPS_eCharpos
        Until GPS_eCharpos = SizeOf(GPS_gsv3RxBuffer) ' Repeat the loop until the buffer runs out
        '-----------------------------------------------------------------------
        F_ErrorGSV3file = 1                           ' If the Frame is bad do nothing.
        GoTo ReadGSV3exit                             '  GoTo error.

SuccesReadingGSV3:
        ' Check the final checksum of file.
        GoSub MakeFinalChecksum_Sub
        If DO_Checksum <> FinalLineChecksum Then     ' Compare the 2 checksums.
            F_ErrorGSV3file = 1
        EndIf
ReadGSV3exit:
        If F_ErrorGSV3file = 1 Then
            HRSOut "$GPGSV-3 fails",13,10
            $ifdef _SynchronizedOperation_
            GoTo DetectStartFile
            $endif
        Else
            F_GSV3_fileOK = 1        
        EndIf
    EndIf

    '---Load Fourth GSV message-------------------------------------------------- 
    ' This message transfers information about satellites in view.
    ' Each record contains the information for up to 4 channels, allowing up to 12
    ' satellites in view. In the final record of the sequence the unused channel fields
    ' are left blank with commas to indicate that a field has been omitted.
    ' $GPGSV
    ' Input     : RsIn {__GPS_GSV_Timeout},Wait("$GPGSV") 
    ' Output    : GPS_gsv4RxBuffer
    ' Notes     : None
    '
    If NumberOfmessages > 3 Then
        F_GSV4_fileOK = 0                      ' Initialize to print the GSV4 file.
        F_Do_CKS = 0
        GPS_eCharpos = 0
        DO_Checksum = cvGSV_Checksum           ' Load checksum GPGSV word.
        GPS_wFSR0 = VarPtr GPS_gsv4RxBuffer    ' Point FSR0L/H to the start of the buffer

        RsIn {__GPS_GSV_Timeout},Wait("$GPGSV")     ' Wait for the $GPGSV string (with a timeout)
        ' Receive the rest of the NMEA text into GPS_gsv3RxBuffer
        Repeat                                      ' Create a loop to receive the serial string
            RsIn {__GPS_GSV_Timeout},WREG           ' Receive a character serially from the GPS module
            POSTINC0 = WREG                         ' Load a value of array GPS_gsv3RxBuffer        
            GPS_bChar = WREG
            ' Make checksum of all of bytes of the file until "*" character.
            If F_Do_CKS = 0 Then
                If GPS_bChar <> "*" Then             
                    DO_Checksum = DO_Checksum ^ GPS_bChar ' Do checksum
                Else
                    F_Do_CKS = 1                    ' Stop DO checksum.
                EndIf
            EndIf
            If GPS_bChar = 13 Then GoTo SuccesReadingGSV4  ' Exit the loop when the string reaches the end
            Inc GPS_eCharpos
        Until GPS_eCharpos = SizeOf(GPS_gsv4RxBuffer) ' Repeat the loop until the buffer runs out
        '-----------------------------------------------------------------------
        F_ErrorGSV4file = 1                           ' If the Frame is bad do nothing.
        GoTo ReadGSV4exit                             '  GoTo error.

SuccesReadingGSV4:
        ' Check the final checksum of file.
        GoSub MakeFinalChecksum_Sub
        If DO_Checksum <> FinalLineChecksum Then     ' Compare the 2 checksums.
            F_ErrorGSV4file = 1
        EndIf
ReadGSV4exit:
        If F_ErrorGSV4file = 1 Then
            HRSOut "$GPGSV-4 fails",13,10
        Else
            F_GSV4_fileOK = 1        
        EndIf
    EndIf
    $ifdef _SynchronizedOperation_
    GoTo DetectStartFile
    $endif
    '---End load GSV messages--------------------------------------------------- 
$endif
'*******************************************************************************
' Loading the "$GPRMC" file - Recommended minimum specific GPS/Transit data
' Find the date of the GPS fix
' Input     : RsIn {__GPS_RMC_Timeout},WREG
' Output    : GPS_rmcRxBuffer
' Notes     : None
'
ReadRMC_Sub:
    F_RMC_fileOK = 0                    ' Initialize to print the RMC file.
    F_Do_CKS = 0
    DO_Checksum = cvRMC_Checksum        ' Load checksum GPRMC word.
    GPS_eCharpos = 0
    GPS_wFSR0 = VarPtr GPS_rmcRxBuffer  ' Point FSR0L/H to the start of the buffer

    $ifndef _SynchronizedOperation_
    RsIn {__GPS_RMC_Timeout},Wait("$GPRMC") ' Wait for the $GPRMC string (with a timeout)
    $endif
    ' Receive the rest of the NMEA text into GPS_rmcRxBuffer
    Repeat                              ' Create a loop to receive the serial string
        RsIn {__GPS_RMC_Timeout},WREG    ' Receive a character serially from the GPS module
        POSTINC0 = WREG                 ' Load a value of array GPS_rmcRxBuffer        
        GPS_bChar = WREG
        ' Make checksum of all of bytes of the file until "*" character.
        If F_Do_CKS = 0 Then
            If GPS_bChar <> "*" Then             
                DO_Checksum = DO_Checksum ^ GPS_bChar ' Do checksum
            Else
                F_Do_CKS = 1                    ' Stop DO checksum.
            EndIf
        EndIf
        If GPS_bChar = 13 Then GoTo SuccesReadingRMC         ' Exit the loop when the string reaches the end
        Inc GPS_eCharpos
    Until  GPS_eCharpos = SizeOf(GPS_rmcRxBuffer)    ' Repeat the loop until the buffer runs out
    '---------------------------------------------------------------------------
    F_ErrorRMCfile = 1                               ' Get an error, the Carriage Return not found.
    GoTo ReadRMCexit                                 ' If the Frame is bad GoTo error.
SuccesReadingRMC:

    ' Check the final checksum of file.
    GoSub MakeFinalChecksum_Sub
    If DO_Checksum <> FinalLineChecksum Then      ' Compare the 2 checksums.
        F_ErrorRMCfile = 1                        ' Get an error, checksum fails.
    EndIf
ReadRMCexit:
    If F_ErrorRMCfile = 1 Then
        HRSOut "$GPRMC fails",13,10
    Else
        F_RMC_fileOK = 1
    EndIf
    $if _defined(_SynchronizedOperation_) And _defined(_ReadGPVTG_) 
    GoTo DetectStartFile
    $endif
'*******************************************************************************
' Find the information about Velocity, course over ground, and ground speed.
' Input     : RsIn {__GPS_VTG_Timeout},WREG
' Output    : cvVTG_Checksum
' Notes     : None
'
$ifdef _ReadGPVTG_
' Loading the $GPVTG file - Track made good and ground speed
ReadVTG_Sub:
    F_VTG_fileOK = 0                   ' Initialize to print the VTG file.
    F_Do_CKS = 0
    GPS_eCharpos = 0
    DO_Checksum = cvVTG_Checksum       ' Load checksum GPGGA word.
    GPS_wFSR0 = VarPtr GPS_vtgRxBuffer ' Point FSR0L/H to the start of the buffer

    $ifndef _SynchronizedOperation_
    RsIn {__GPS_VTG_Timeout},Wait("$GPVTG") ' Wait for the $GPVTG string (with a timeout)
    $endif
    ' Receive the rest of the NMEA text into GPS_vtgRxBuffer
    Repeat                                      ' Create a loop to receive the serial string
        RsIn {__GPS_VTG_Timeout},WREG            ' Receive a character serially from the GPS module
        POSTINC0 = WREG                         ' Load a value of array GPS_vtgRxBuffer        
        GPS_bChar = WREG
        ' Make checksum of all of bytes of the file until "*" character.
        If F_Do_CKS = 0 Then
            If GPS_bChar <> "*" Then             
                DO_Checksum = DO_Checksum ^ GPS_bChar ' Do checksum
            Else
                F_Do_CKS = 1                    ' Stop DO checksum.
            EndIf
        EndIf
        If GPS_bChar = 13 Then GoTo SuccesReadingVTG ' Exit the loop when the string reaches the end
        Inc GPS_eCharpos
    Until GPS_eCharpos = SizeOf(GPS_vtgRxBuffer)  ' Repeat the loop until the buffer runs out
    '---------------------------------------------------------------------------
    F_ErrorVTGfile = 1                            ' Get an error, the Carriage Return not found.
    GoTo ReadVTGexit                              ' If the Frame is bad GoTo error.

SuccesReadingVTG:
    ' Check the final checksum of file.
    GoSub MakeFinalChecksum_Sub
    If DO_Checksum <> FinalLineChecksum Then     ' Compare the 2 checksums.
        F_ErrorVTGfile = 1
    EndIf

ReadVTGexit:
    If F_ErrorVTGfile = 1 Then
        HRSOut "$GPVTG fails",13,10
    Else
        F_VTG_fileOK = 1
    EndIf
$endif
'===============================================================================
EndOfReadingFiles:
$ifdef _ResetAmicus18Terminal_
    HRSOut 1                            ' Clear the serial terminal
$endif
'===============================================================================
' NOW all GPS files have been recirded then send the lines to the terminal
' if this function is enabled by the preprocessor at compiler time.
PrintBuffers:
(*
$GPGGA
GGA essential fix data which provide 3D location and accuracy data.
    $GPGGA,123519,4807.038,N,01131.000,E,1,08,0.9,545.4,M,46.9,M,,*47
    Where:
    GGA Global Positioning System Fix Data
    123519 Fix taken at 12:35:19 UTC
    4807.038,N Latitude 48 deg 07.038' N
    01131.000,E Longitude 11 deg 31.000' E
    1 Fix quality: 0 = invalid
    1 = GPS fix (SPS)
    2 = DGPS fix
    3 = PPS fix
    4 = Real Time Kinematic
    5 = Float RTK
    6 = estimated (dead reckoning) (2.3 feature)
    7 = Manual input mode
    8 = Simulation mode
    08 Number of satellites being tracked
    0.9 Horizontal dilution of position
    545.4,M Altitude, Meters, above mean sea level
    46.9,M Height of geoid (mean sea level) above WGS84
    ellipsoid
    (empty field) time in seconds since last DGPS update
    (empty field) DGPS station ID number
    *47 the checksum data, always begins with *
*)
' "$GPGGA" FILE
'---Print GPGGA-----------------------------------------------------------------
' Input     : GPS_ggaRxBuffer
' Output    : Terminal
' Notes     : None
'
    $ifdef _PrintGPGGA_
    ' Print the "$GPGGA" GPS_ggaRxBuffer.
    If F_ErrorGGAfile = 0 Then  ' The gga file is good then enable to send the file to the terminal.
        GPS_wFSR0 = VarPtr GPS_ggaRxBuffer  ' Point FSR0L/H to the start of the buffer
        HRSOut "$GPGGA"
        Repeat                  ' Create a Loop To read the buffer String
            WREG = POSTINC0     ' Read the array GPS_ggaRxBuffer
            HRSOut WREG
        Until WREG = 13         ' Repeat the Loop when the String reaches the End
    EndIf                       ' because I know that the Carriage Return is already written.
    $endif                              
'$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
(*
$GPGSA
GSA  GPS DOP and active satellites
    $GPGSA,A,3,04,05,,09,12,,,24,,,,,2.5,1.3,2.1*39
    Where:
    GSA Satellite status
    A Auto selection of 2D or 3D fix (M = manual)
    3 3D fix  values include: 1 = no fix
    2 = 2D fix
    3 = 3D fix
    04,05 PRNs of satellites used for fix (space for 12)
    2.5 PDOP (dilution of precision)
    1.3 Horizontal dilution of precision (HDOP)
    2.1 Vertical dilution of precision (VDOP)
    *39 the checksum data, always begins with *
*)
' "$GPGSA" FILE
'---Print GPGGA-----------------------------------------------------------------
' Input     : GPS_gsaRxBuffer
' Output    : Terminal
' Notes     : None
'
$ifdef _ReadGPGSA_
    $ifdef _PrintGPGSA_
    ' Print the "$GPGSA" GPS_ggaRxBuffer.
    If F_GSA_fileOK = 1 Then    ' The gsa file is good then enable to send the file to the terminal.
        GPS_wFSR0 = VarPtr GPS_gsaRxBuffer  ' Point FSR0L/H to the start of the buffer
        HRSOut "$GPGSA"
        Repeat                  ' Create a Loop To read the buffer String
            WREG = POSTINC0     ' Read the array GPS_ggaRxBuffer
            HRSOut WREG
        Until WREG = 13         ' Repeat the Loop when the String reaches the End
    EndIf                       ' because I know that the Carriage Return is already written.
    $endif                      
$endif
'$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
(*
"$GPGSV" FILE
    GSV  Satellites in View
    $GPGSV,2,1,08,01,40,083,46,02,17,308,41,12,07,344,39,14,22,228,45*75
    Where: GSV Satellites in view
    2 Number of sentences for full data
    1 sentence 1 of 2
    08 Number of satellites in view
    01 Satellite PRN number
    40 Elevation, degrees
    083 Azimuth, degrees
    46 SNR  higher is better
    for up to 4 satellites per sentence
    *75 the checksum data, always begins with *
*)
'---Print GPGSV-1---------------------------------------------------------------
' Input     : GPS_gsv1RxBuffer
' Output    : Terminal
' Notes     : None
' Print the "$GPGSV" GPS_gsv1RxBuffer.
'
$ifdef _ReadGPGSV_
    $ifdef _PrintGPGSV_
    '---------------------------------------------------------------------------
    If F_GSV1_fileOK = 1 Then   ' The gsv1 file is good then enable to send the file to the terminal.
        GPS_wFSR0 = VarPtr GPS_gsv1RxBuffer    ' Point FSR0L/H to the start of the buffer
        HRSOut "$GPGSV"
        Repeat                  ' Create a Loop To read the buffer String
            WREG = POSTINC0     ' Read the array GPS_ggaRxBuffer
            HRSOut WREG
        Until WREG = 13         ' Repeat the Loop when the String reaches the End
    EndIf
'---Print GPGSV-2--------------------------------------------------------------
' Input     : GPS_gsv2RxBuffer
' Output    : Terminal
' Notes     : None
' Print the "$GPGSV" GPS_gsv2RxBuffer.
'
    If F_GSV2_fileOK = 1 Then   ' The gsv2 file is good then enable to send the file to the terminal.
        GPS_wFSR0 = VarPtr GPS_gsv2RxBuffer    ' Point FSR0L/H to the start of the buffer
        HRSOut "$GPGSV"
        Repeat                  ' Create a Loop To read the buffer String
            WREG = POSTINC0     ' Read the array GPS_ggaRxBuffer
            HRSOut WREG
        Until WREG = 13         ' Repeat the Loop when the String reaches the End
    EndIf                       ' because I know that the Carriage Return is already written.
'---Print GPGSV-3---------------------------------------------------------------
' Input     : GPS_gsv3RxBuffer
' Output    : Terminal
' Notes     : None
' Print the "$GPGSV" GPS_gsv3RxBuffer.
'
    If F_GSV3_fileOK = 1 Then   ' The gsv3 file is good then enable to send the file to the terminal.
        GPS_wFSR0 = VarPtr GPS_gsv3RxBuffer    ' Point FSR0L/H to the start of the buffer
        HRSOut "$GPGSV"
        Repeat                  ' Create a Loop To read the buffer String
            WREG = POSTINC0     ' Read the array GPS_ggaRxBuffer
            HRSOut WREG
        Until WREG = 13         ' Repeat the Loop when the String reaches the End
    EndIf                       ' because I know that the Carriage Return is already written.

'---Print GPGSV-4---------------------------------------------------------------
' Input     : GPS_gsv4RxBuffer
' Output    : Terminal
' Notes     : None
' Print the "$GPGSV" GPS_gsv4RxBuffer.
'
    If F_GSV4_fileOK = 1 Then   ' The gsv4 file is good then enable to send the file to the terminal.
        GPS_wFSR0 = VarPtr GPS_gsv4RxBuffer    ' Point FSR0L/H to the start of the buffer
        HRSOut "$GPGSV"
        Repeat                  ' Create a Loop To read the buffer String
            WREG = POSTINC0     ' Read the array GPS_ggaRxBuffer
            HRSOut WREG
        Until WREG = 13         ' Repeat the Loop when the String reaches the End
    EndIf                       ' because I know that the Carriage Return is already written.
    '---------------------------------------------------------------------------
    $endif 
$endif
'$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
(*
"$GPRMC" FILE
RMC  NMEA has its own version of essential gps pvt (position, velocity, time) data
    $GPRMC,123519,A,4807.038,N,01131.000,E,022.4,084.4,230394,003.1,W*6A
    Where:
    RMC Recommended Minimum sentence C
    123519 Fix taken at 12:35:19 UTC
    A Status A=active or V=Void.
    4807.038,N Latitude 48 deg 07.038' N
    01131.000,E Longitude 11 deg 31.000' E
    022.4 Speed over the ground in knots
    084.4 Track angle in degrees True
    230394 Date  23rd of March 1994
    003.1,W Magnetic Variation
    *6A The checksum data, always begins with *
*)
'---Print GPRMC-----------------------------------------------------------------
' Input     : GPS_rmcRxBuffer
' Output    : Terminal
' Notes     : None
' Print the "$GPRMC" GPS_rmcRxBuffer.
'
    $ifdef _PrintGPRMC_
    If F_RMC_fileOK = 1 Then    ' The rmc file is good then enable to send the file to the terminal.
        GPS_wFSR0 = VarPtr GPS_rmcRxBuffer    ' Point FSR0L/H to the start of the buffer
        HRSOut "$GPRMC"
        Repeat                  ' Create a Loop To read the buffer String
            WREG = POSTINC0     ' Read the array GPS_ggaRxBuffer
            HRSOut WREG
        Until WREG = 13         ' Repeat the Loop when the String reaches the End
    EndIf                       ' because I know that the Carriage Return is already written.
    $endif                      
'$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
(*
"$GPVTG" FILE
    VTG  Velocity made good.
    $GPVTG,054.7,T,034.4,M,005.5,N,010.2,K*48
    where:
    VTG Track made good and ground speed
    054.7,T True track made good (degrees)
    034.4,M Magnetic track made good
    005.5,N Ground speed, knots
    010.2,K Ground speed, Kilometers per hour
    *48 Checksum
*)
'---PrintGPVTG------------------------------------------------------------------
' Input     : GPS_vtgRxBuffer
' Output    : Terminal
' Notes     : None
' Print the "$GPVTG" GPS_vtgRxBuffer.
'
$ifdef _ReadGPVTG_
    $ifdef _PrintGPVTG_
    If F_VTG_fileOK = 1 Then    ' The vtg file is good then enable to send the file to the terminal.
        GPS_wFSR0 = VarPtr GPS_vtgRxBuffer  ' Point FSR0L/H to the start of the buffer
        HRSOut "$GPVTG"
        Repeat                  ' Create a Loop To read the buffer String
            WREG = POSTINC0     ' Read the array GPS_ggaRxBuffer
            HRSOut WREG
        Until WREG = 13         ' Repeat the Loop when the String reaches the End
    EndIf                       ' because I know that the Carriage Return is already written.
    $endif                     
$endif
'===============================================================================
' Place the time and position values into their respective variables
' Input     : GPS_ggaRxBuffer
' Output    : Variables
' Notes     : None
'
GetGPGGA:
    If F_ErrorGGAfile = 1 Then GoTo GetGPGGAexit
' ("$GPGGA")
    GPS_ExtractValueGGA(1)    ' Get time ASCII value into GPS_bDataString
    GoSub _GPS_ElementsToInt_Sub                                ' Convert the lements into integer values
    GPS_Hours = GPS_bDataString#1 + (GPS_bDataString#0 * 10)    ' Extract the Hours value
    GPS_Minutes = GPS_bDataString#3 + (GPS_bDataString#2 * 10)  ' Extract the Minutes value
    GPS_Seconds = GPS_bDataString#5 + (GPS_bDataString#4 * 10)  ' Extract the Seconds value  
'-------------------------------------------------------------------------------    
' Get the latitude
    GPS_ExtractValueGGA(3)          ' Get the latitude qGPS_AntBearing ASCII value into GPS_bDataString
    GPS_LatBearing = GPS_bDataString#0  

    GPS_ExtractValueGGA(2)          ' Get the latitude ASCII value into GPS_bDataString
    ' Get Format DM.m    
    F_GetSign = 0                   ' DO not read the sign. (Save time)
    GoSub GPS_StrToFloat_Sub        ' Convert the latitude ASCII value into a floating point variable
    GPS_LAT_DMm = Atf_fResult

    '---------------------------------------------------------------------------
    ' Get Format D.d (another mode using the GPS DM.m Float value)
    GPS_TempDword = Atf_fResult / 100                  ' Get an integer.
    Atf_fResult = Atf_fResult - (GPS_TempDword * 100)  ' Subtract the degrees
    GPS_LAT_Dd = (Atf_fResult / 60) + GPS_TempDword    ' Conver the seconds to decimal degrees + Degrees.
    If GPS_LatBearing = "S" Then
        GPS_LAT_Dd = -GPS_LAT_Dd
    EndIf

    '---------------------------------------------------------------------------
    ' Get Format DMS (another mode using the GPS DM.m Float value)
    GPS_LAT_Degrees_dms = GPS_LAT_DMm / 100                 ' Integer Degrees 
    Atf_fResult = GPS_LAT_DMm - (GPS_LAT_Degrees_dms * 100) ' Remaining Float minutes.
    GPS_LAT_Minutes_dms = Atf_fResult                       ' Integer minutes
    Atf_fResult = Atf_fResult - GPS_LAT_Minutes_dms         ' Remaining Float seconds.
    GPS_LAT_Seconds_dms = Atf_fResult * 60                  ' Float seconds

    '---------------------------------------------------------------------------
'    ' Get Format DMS
'    GoSub _GPS_ElementsToInt_Sub    ' Convert the lements into integer values
'    ' Get the Latitude ASCII value into GPS_bDataString
'    GPS_LAT_Degrees_dms = GPS_bDataString#1 + (GPS_bDataString#0 * 10)
'    GPS_LAT_Minutes_dms = GPS_bDataString#3 + (GPS_bDataString#2 * 10)
'    GPS_TempDword = (GPS_bDataString#5 * 1000) + (GPS_bDataString#6 * 100) + (GPS_bDataString#7 * 10) + GPS_bDataString#8 
'    ' The Seconds are Float to see the decimals
'    GPS_LAT_Seconds_dms = (GPS_TempDword * 60) / 10000  ' Result = Integer Seconds
'    GPS_TempDword = (GPS_TempDword * 60) // 10000       ' Extract the Seconds decimals
'    GPS_LAT_Seconds_dms = GPS_LAT_Seconds_dms + (GPS_TempDword / 100000) ' And sum the decimals To the Seconds Float 

    '--------------------------------------------------------------------------- 
'    ' Get Format D.d
'    'GPS_TempDword = (GPS_bDataString#5 * 1000) + (GPS_bDataString#6 * 100) + (GPS_bDataString#7 * 10) + (GPS_bDataString#8) ' 0,Minutes  
'    'GPS_LAT_Dd = GPS_TempDword / 600000
'    GPS_LAT_Dd = (GPS_bDataString#5 * 1000) + (GPS_bDataString#6 * 100) + (GPS_bDataString#7 * 10) + (GPS_bDataString#8) ' 0,Minutes  
'    GPS_LAT_Dd = GPS_LAT_Dd / 600000
'    GPS_LAT_Dd = GPS_LAT_Dd + (GPS_bDataString#3 + (GPS_bDataString#2 * 10)) / 60   ' Minutes
'    GPS_LAT_Dd = GPS_LAT_Dd + GPS_bDataString#1 + (GPS_bDataString#0 * 10)          ' Degrees
'    If GPS_LatBearing = "S" Then
'        GPS_LAT_Dd = -GPS_LAT_Dd
'    EndIf
'-------------------------------------------------------------------------------    
' Get the longitude
    GPS_ExtractValueGGA(5)                ' Get the longitude qGPS_AntBearing ASCII value into GPS_bDataString
    GPS_LonBearing = GPS_bDataString#0  

    GPS_ExtractValueGGA(4)                ' Get the longitude ASCII value into GPS_bDataString
GetFormatDMm:
    ' Get Format DM.m
    F_GetSign = 0                   ' DO not read the sign. (Save time)
    GoSub GPS_StrToFloat_Sub
    GPS_LON_DMm = Atf_fResult       ' Convert the longitude ASCII value into a floating point variable

    '---------------------------------------------------------------------------
    ' Get Format D.d (another mode using the GPS DM.m float value)
    GPS_TempDword = Atf_fResult / 100                   ' Get an integer.
    Atf_fResult = Atf_fResult - (GPS_TempDword * 100)   ' Subtract the degrees
    GPS_LON_Dd = (Atf_fResult / 60) + GPS_TempDword     ' Conver the seconds to decimal degrees + Degrees.
    If GPS_LonBearing = "W" Then
        GPS_LON_Dd = -GPS_LON_Dd
    EndIf

    '---------------------------------------------------------------------------
    ' Get Format DMS (another mode using the GPS DM.m Float value)
    GPS_LON_Degrees_dms = GPS_LON_DMm / 100                 ' Integer Degrees 
    Atf_fResult = GPS_LON_DMm - (GPS_LON_Degrees_dms * 100) ' Remaining float minutes.
    GPS_LON_Minutes_dms = Atf_fResult                       ' Integer minutes
    Atf_fResult = Atf_fResult - GPS_LON_Minutes_dms         ' Remaining float seconds.
    GPS_LON_Seconds_dms = Atf_fResult * 60                  ' Float seconds

    '---------------------------------------------------------------------------
GetFormatDMS:
'    ' Get Format DMS
'    GoSub _GPS_ElementsToInt_Sub    ' Convert the elements into integer values
'    ' Get the Longitude ASCII value into GPS_bDataString
'    GPS_LON_Degrees_dms = (GPS_bDataString#0 * 100) + (GPS_bDataString#1 * 10) + GPS_bDataString#2
'    GPS_LON_Minutes_dms = GPS_bDataString#4 + (GPS_bDataString#3 * 10)

'    GPS_TempDword = (GPS_bDataString#6 * 1000) + (GPS_bDataString#7 * 100) + (GPS_bDataString#8 * 10) + GPS_bDataString#9 
'    ' The Seconds are Float to see the decimals
'    GPS_LON_Seconds_dms = (GPS_TempDword * 60) / 10000  ' Result = Integer Seconds
'    GPS_TempDword = (GPS_TempDword * 60) // 10000       ' Extract the Seconds decimals 
'    GPS_LON_Seconds_dms = GPS_LON_Seconds_dms + (GPS_TempDword / 100000) ' And sum the decimals to the Seconds Float      
    '---------------------------------------------------------------------------
GetFormatDd:
    ' Get Format D.d
    'GPS_TempDword = (GPS_bDataString#6 * 1000) + (GPS_bDataString#7 * 100) + (GPS_bDataString#8 * 10) + GPS_bDataString#9 
    'GPS_LON_Dd = GPS_TempDword / 600000
'    GPS_LON_Dd = (GPS_bDataString#6 * 1000) + (GPS_bDataString#7 * 100) + (GPS_bDataString#8 * 10) + GPS_bDataString#9 
'    GPS_LON_Dd = GPS_LON_Dd / 600000
'    GPS_LON_Dd = GPS_LON_Dd + (GPS_bDataString#4 + (GPS_bDataString#3 * 10)) / 60
'    GPS_LON_Dd = GPS_LON_Dd + GPS_bDataString#2 + (GPS_bDataString#1 * 10) + (GPS_bDataString#0 * 100)        
'    If GPS_LonBearing = "W" Then
'        GPS_LON_Dd = -GPS_LON_Dd
'    EndIf
'-------------------------------------------------------------------------------    
    GPS_ExtractValueGGA(6)                              ' Get the fix type ASCII value into GPS_bDataString
    GPS_FixType = GPS_bDataString#0 - 48                ' Convert it into an integer   

    ' Satellites in view:
    GPS_ExtractValueGGA(7)                              ' Get the amount of satellites ASCII value into GPS_bDataString
    GPS_SatsInView = (GPS_bDataString#0 - 48) * 10              ' Convert it into an integer
    GPS_SatsInView = GPS_SatsInView + (GPS_bDataString#1 - 48)  ' Convert it into an integer
    ' Another alternative
'    GoSub _GPS_ElementsToInt                              ' Convert the elements into integer values
'    GPS_SatsInView = GPS_bDataString#0 + (GPS_bDataString#1 * 10) ' Extract the amount of satellites

    $ifdef _Altitude_
    GPS_ExtractValueGGA(9)          ' Get Altitude
    F_GetSign = 0                   ' DO not read the sign. (Save time)
    GoSub GPS_StrToFloat_Sub        ' Convert the Altitude ASCII value into a floating point variable
    Atf_fResult = Atf_fResult + 0.5 ' Rounding to high value.
    GPS_Altitude = Atf_fResult      ' Save in Word variable to print the whole value only.
    $endif

GetGPGGAexit:
'===============================================================================
' Place the time and position values into their respective variables
' Input     : GPS_rmcRxBuffer
' Output    : Variables
' Notes     : None
'
GetGPRMC:
(*
$GPRMC
    Recommended minimum specific GPS/Transit data
    eg1. $GPRMC,081836,A,3751.65,S,14507.36,E,000.0,360.0,130998,011.3,E*62
    eg2. $GPRMC,225446,A,4916.45,N,12311.12,W,000.5,054.7,191194,020.3,E*68
               225446       Time of fix 22:54:46 UTC
               A            Navigation receiver warning A = OK, V = warning
               4916.45,N    Latitude 49 deg. 16.45 min North
               12311.12,W   Longitude 123 deg. 11.12 min West
               000.5        Speed over ground, Knots
               054.7        Course Made Good, True
               191194       Date of fix  19 November 1994
               020.3,E      Magnetic variation 20.3 deg East
               *68          mandatory checksum
    
    eg3. $GPRMC,220516,A,5133.82,N,00042.24,W,173.8,231.8,130694,004.2,W*70
                  1    2   3     4    5     6    7    8      9    10   11 12
          1   220516     Time Stamp
          2   A          validity - A-ok, V-invalid
          3   5133.82    current Latitude
          4   N          North/South
          5   00042.24   current Longitude
          6   W          East/West
          7   173.8      Speed in knots
          8   231.8      True course
          9   130694     Date Stamp
          10  004.2      Variation
          11  W          East/West
          12  *70        checksum
    
    eg4. $GPRMC,hhmmss.ss,A,llll.ll,a,yyyyy.yy,a,x.x,x.x,ddmmyy,x.x,a*hh
    1    = UTC of position fix
    2    = Data status (V=navigation receiver warning)
    3    = Latitude of fix
    4    = N or S
    5    = Longitude of fix
    6    = E or W
    7    = Speed over ground in knots
    8    = Track made good in degrees True
    9    = UT date
    10   = Magnetic variation degrees (Easterly var. subtracts from true course)
    11   = E or W
    12   = Checksum
*)
' Place the date information into its respective variables
' ("$GPRMC")   
    If F_RMC_fileOK = 1 Then
        GPS_ExtractValueRMC(2)    ' Extract position 2: A = Data STATUS (V = navigation receiver warning)
        GPS_RMC_Valid = GPS_bDataString#0    ' A (ASCII = Valid) V (ASCII = Varning)
    
        GPS_ExtractValueRMC(9)    ' Extract the date information into the GPS_bDataString array
        GoSub _GPS_ElementsToInt_Sub    ' Convert the elements into integer values
        GPS_Day = GPS_bDataString#1 + (GPS_bDataString#0 * 10)     ' Extract the Day value
        GPS_Month = GPS_bDataString#3 + (GPS_bDataString#2 * 10)   ' Extract the Month value
        GPS_Year = GPS_bDataString#5 + (GPS_bDataString#4 * 10)    ' Extract the Year value
    EndIf
'-------------------------------------------------------------------------------
'===============================================================================
' Place the time and position values into their respective variables
' Input     : GPS_vtgRxBuffer
' Output    : Variables
' Notes     : None
'
(*
$GPVTG
    Track Made Good and Ground Speed.
    eg1. $GPVTG,360.0,T,348.7,M,000.0,N,000.0,K*43
    eg2. $GPVTG,054.7,T,034.4,M,005.5,N,010.2,K
               054.7,T      True track made good
               034.4,M      Magnetic track made good
               005.5,N      Ground speed, knots
               010.2,K      Ground speed, Kilometers per hour
    
    eg3. $GPVTG,t,T,,,s.ss,N,s.ss,K*hh
    1    = Track made good
    2    = Fixed text 'T' indicates that track made good is relative to true north
    3    = not used
    4    = not used
    5    = Speed over ground in knots
    6    = Fixed text 'N' indicates that speed over ground in in knots
    7    = Speed over ground in kilometers/hour
    8    = Fixed text 'K' indicates that speed over ground is in kilometers/hour
    9    = Checksum
    
    The actual track made good and speed relative to the ground.
    $--VTG,x.x,T,x.x,M,x.x,N,x.x,K
    x.x,T = Track, degrees True
    x.x,M = Track, degrees Magnetic
    x.x,N = Speed, knots
    x.x,K = Speed, Km/hr
*) 
GetGPVTG:
' Place the date information into its respective variables
' ("$GPVTG")
    $ifdef _ReadGPVTG_
    If F_VTG_fileOK = 1 Then
        ' Extract the Measured heading in degrees.
        GPS_ExtractValueVTG(1)      ' Extract the date information into the GPS_bDataString array
        F_GetSign = 0               ' DO not read the sign. (save time)
        GoSub GPS_StrToFloat_Sub    ' Convert the data to Float variable.
        GPS_Heading = Atf_fResult   ' Load to a float variable.
    
        ' Extract the speed in KM/H.
        GPS_ExtractValueVTG(7)      ' Extract the date information into the GPS_vtgDataString array
        F_GetSign = 0               ' DO not read the sign. (Save time)
        GoSub GPS_StrToFloat_Sub    ' Convert the data to Float variable.
        GPS_SpeedKm = Atf_fResult   ' Convert the Speed to integer.
    EndIf
    $endif
'===============================================================================

    Return  ' Return of GPS_Parse() routine.
'###############################################################################
'===============================================================================
' Come here if a software UART serial timeout occurs.
' Input     : Loading buffers code
' Output    : exit the GPS_Parse() routine.
' Notes     : Every Time Out error has a special message.
'
__GPS_GGA_Timeout:
    HRSOut "GPS GGA TimeOut",13,10
    Clear GPS_ggaRxBuffer
    GPS_ggaRxBuffer#0 = 13  ' Because there is not Carriage Return at start up.  
    GoTo ExitTimeOut  

__GPS_StartTimeout:
    HRSOut "GPS START TimeOut",13,10
    $ifdef _ResetAmicus18Terminal_
    HRSOut 1                            ' Clear the serial terminal
    $endif
    GoTo ExitClearAll    

__GPS_GSA_Timeout:
    $ifdef _ReadGPGSA_
    HRSOut "GPS GSA TimeOut",13,10
    Clear GPS_gsaRxBuffer
    GPS_gsaRxBuffer#0 = 13  ' Because there is not Carriage Return at start up.
    GoTo ExitTimeOut 
    $endif

__GPS_GSV_Timeout:
    $ifdef _ReadGPGSV_ 
    HRSOut "GPS GSV TimeOut",13,10
    Clear GPS_gsv1RxBuffer    
    Clear GPS_gsv2RxBuffer   
    Clear GPS_gsv3RxBuffer   
    GPS_gsv1RxBuffer#0 = 13  ' Because there is not Carriage Return at start up.
    GPS_gsv2RxBuffer#0 = 13  ' Because there is not Carriage Return at start up.
    GPS_gsv3RxBuffer#0 = 13  ' Because there is not Carriage Return at start up.        
    GoTo ExitTimeOut
    $endif  

__GPS_RMC_Timeout:
    HRSOut "GPS RMC TimeOut",13,10
    Clear GPS_rmcRxBuffer
    GPS_rmcRxBuffer#0 = 13  ' Because there is not Carriage Return at start up.  
    GoTo ExitTimeOut 

__GPS_VTG_Timeout:
    $ifdef _ReadGPVTG_
    HRSOut "GPS VTG TimeOut",13,10
    Clear GPS_vtgRxBuffer
    GPS_vtgRxBuffer#0 = 13  ' Because there is not Carriage Return at start up.
    $endif

ExitTimeOut:
    GPS_FixType = 0                 ' Indicate no satellites in view
    GPS_Flags = 0
    GPS_FlagsError = 0
    GPS_FilesOKFlags = 0
    $ifdef _ResetAmicus18Terminal_
    HRSOut 1                            ' Clear the serial terminal
    $endif
    Return  ' Return of GPS_Parse() routine.
'===============================================================================        
ExitClearAll:
    GPS_FixType = 0                 ' Indicate no satellites in view
    GPS_Flags = 0
    GPS_FlagsError = 0
    GPS_FilesOKFlags = 0
    ' This way the Strings buffers have always a zero after the CR,LF characters.
    ' In this case PIC time is saved working with loops.
    Clear GPS_ggaRxBuffer
    GPS_ggaRxBuffer#0 = 13  ' Because there is not Carriage Return at start up.    
    Clear GPS_rmcRxBuffer
    GPS_rmcRxBuffer#0 = 13  ' Because there is not Carriage Return at start up.        
    $ifdef _ReadGPGSA_
    Clear GPS_gsaRxBuffer
    GPS_gsaRxBuffer#0 = 13  ' Because there is not Carriage Return at start up.
    $endif
    $ifdef _ReadGPVTG_
    Clear GPS_vtgRxBuffer
    GPS_vtgRxBuffer#0 = 13  ' Because there is not Carriage Return at start up.
    $endif
    $ifdef _ReadGPGSV_    
    Clear GPS_gsv1RxBuffer    
    Clear GPS_gsv2RxBuffer   
    Clear GPS_gsv3RxBuffer   
    GPS_gsv1RxBuffer#0 = 13  ' Because there is not Carriage Return at start up.
    GPS_gsv2RxBuffer#0 = 13  ' Because there is not Carriage Return at start up.
    GPS_gsv3RxBuffer#0 = 13  ' Because there is not Carriage Return at start up.        
    $endif        
    '---------------------------------------------------------------------------
    Return
'###############################################################################
' Some usefull routines for your libraries.
' Input     : ASCCI or HEX byte
' Output    : Convert to HEX or ASCII Byte.
' Notes     : Using Local Variables library (DeclareLocalVars.Inc).
'
    LocalByte0(ASCII_Char)

$define ConvertAsciiToHex(pValue, pReturn) '
    ASCII_Char = pValue                    '
    _mConvertAsciiToHex                    '
    pReturn = WREG
     
_mConvertAsciiToHex Macro-
    GoSub ASCIIhex_to_HEXbin_Sub
Endm

#ifMacro- _mConvertAsciiToHex
ASCIIhex_to_HEXbin_Sub:
    ' Transform the ASCII Hex to Bin HEX  from WREG and return the value in WREG.
    If ASCII_Char > 64 Then      
        WREG = ASCII_Char - 55  ' Get (A to F)
    Else
        WREG = ASCII_Char - 48  ' Get (0 to 9)
    EndIf
    WREG = WREG & $0F           ' Make sure that it is one Hex only.
    Return
#endIfMacro-
'===============================================================================
    LocalByte0(HEX_Char)

$define ConvertHexToAscii(pValue, pReturn) '
    WREG = pValue                          '
    _mConvertHexToAscii                    '
    pReturn = WREG

_mConvertHexToAscii Macro-
    GoSub HEXbin_to_ASCIIhex_Sub
Endm

#ifMacro- _mConvertHexToAscii
HEXbin_to_ASCIIhex_Sub:
    ' Transform the Bin HEX to ASCII Hex from WREG and return the value in WREG.
    HEX_Char = WREG & $0F       ' Make sure that it is one Hex only.
    If HEX_Char < 10 Then
        WREG = HEX_Char + 48    ' Get (0 to 9)
    Else
        WREG = HEX_Char + 65    ' Get (A to F)
    EndIf
    Return
#endIfMacro-
'===============================================================================  
'*******************************************************************************
'* Name    : ConfigPNMRX108A(), ConfigPNMRX103A(), ...                         *
'* Purpose : Configure the GPS                                                 *
'* Input   : pOverride = 0=>write config once. pOverride = 0=>write every reset*
'* Output  :                                                                   *
'* Notes   :                                                                   *
'* Syntax  : ConfigPNMRX108A(0), ConfigPNMRX108A(1)                            *
'*******************************************************************************   
' This code does not work with the GPS, but it seems to work with Proteus.
' This code will be usefull for other aplications.
'===============================================================================       
$ifdef _GPS_Configuration_

    LocalWord0(nCommandAddress)
    LocalWord1(nEEP_ADDR)
    LocalByte0(nTXChecksum)
    LocalByte1(nGPSConfig)
    LocalByte2(nGPSCfgPNMRX)
    LocalByte2(nGPS_Char)    

    Symbol vADDR_PNMRX108A = _eeprom - 2
    Symbol vADDR_PNMRX103A = _eeprom - 3
    Symbol vADDR_PNMRX103B = _eeprom - 4
    Symbol vADDR_PNMRX100A = _eeprom - 5
    Symbol vADDR_PNMRX100B = _eeprom - 6
        
' 1 = Override the checking of the eeprom
' 0 = checking the eeprom
'
$define ConfigPNMRX108A(pOverride) '
    nGPSConfig = pOverride         '
    nCommandAddress = PNMRX108A    '
    nEEP_ADDR = vADDR_PNMRX108A    '
    GoSub ConfigPNMRX000_Sub

$define ConfigPNMRX103A(pOverride) '
    nGPSConfig = pOverride         '
    nCommandAddress = PNMRX103A    '
    nEEP_ADDR = vADDR_PNMRX103A    '
    GoSub ConfigPNMRX000_Sub

$define ConfigPNMRX103B(pOverride) '
    nGPSConfig = pOverride         '
    nCommandAddress = PNMRX103B    '
    nEEP_ADDR = vADDR_PNMRX103B    '
    GoSub ConfigPNMRX000_Sub

$define ConfigPNMRX100A(pOverride) '
    nGPSConfig = pOverride         '
    nCommandAddress = PNMRX100A    '
    nEEP_ADDR = vADDR_PNMRX100A    '
    GoSub ConfigPNMRX000_Sub

$define ConfigPNMRX100B(pOverride) '
    nGPSConfig = pOverride         '
    nCommandAddress = PNMRX100B    '
    nEEP_ADDR = vADDR_PNMRX100B    '
    GoSub ConfigPNMRX000_Sub
    
ConfigPNMRX000_Sub:
    If nGPSConfig = 0 Then
        WREG = ERead nEEP_ADDR
        If WREG = $A5 Then Return  ' Is this command already sent?
    EndIf
    nTXChecksum = 0
    RsOut "$"                               ' Send First character of the line.
    Repeat                                  ' Create a loop to receive the serial string
        nGPS_Char = CRead nCommandAddress   ' Read the CData line
        If nGPS_Char = 0 Then 
            RsOut "*"
            Break                           ' Exit the loop when the string reaches the end
        EndIf
        nTXChecksum = nTXChecksum ^ nGPS_Char   ' Make the checksum.
        RsOut nGPS_Char                         ' Send every character.
        Inc nCommandAddress
    Until nGPS_Char = 0

    nGPS_Char = nTXChecksum >> 4 ' Convert the Fist nibble to ASCII.
    If nGPS_Char < 10 Then
        WREG = nGPS_Char + 48    ' Get (0 to 9)
    Else
        WREG = nGPS_Char + 65    ' Get (A to F)
    EndIf
    RsOut WREG

    nGPS_Char = nTXChecksum & $0F ' Convert the Second nibble To ASCII.
    If nGPS_Char < 10 Then
        WREG = nGPS_Char + 48     ' Get (0 to 9)
    Else
        WREG = nGPS_Char + 65     ' Get (A to F)
    EndIf
    RsOut WREG
    RsOut 13,10                     ' End of file.
    EWrite nEEP_ADDR,[$A5]          ' Write the confirmation of writing in eeprom.
    Return
'-------------------------------------------------------------------------------
' Same code but the Checksum is already done by an external calculator.
' Online NMEA XOR checksum calculator: http://www.hhhh.org/wiml/proj/nmeaxor.html
    $define MRX108A 0
    $define MRX103A 1
    $define MRX103B 2
    $define MRX100A 3
    $define MRX100B 4

$define ConfigxPNMRX(pOverride, pPNMRX) '
    nGPSConfig = pOverride              '
    nGPSCfgPNMRX = pPNMRX               '
    nEEP_ADDR = vADDR_PNMRX108A         '
    GoSub ConfigXPNMRX000_Sub

ConfigXPNMRX000_Sub:
    If nGPSConfig = 0 Then
        WREG = ERead nEEP_ADDR
        If WREG = $A5 Then Return  ' Is this command already sent?
    EndIf
    Select nGPSCfgPNMRX
        Case 0
            nEEP_ADDR = vADDR_PNMRX108A
            RsOut CStr XPNMRX108A
        Case 1
            nEEP_ADDR = vADDR_PNMRX103A
            RsOut CStr XPNMRX103A        
        Case 2
            nEEP_ADDR = vADDR_PNMRX103B
            RsOut CStr XPNMRX103B        
        Case 3        
            nEEP_ADDR = vADDR_PNMRX100A
            RsOut CStr XPNMRX100A
        Case 4
            nEEP_ADDR = vADDR_PNMRX100B
            RsOut CStr XPNMRX100B                                  
    EndSelect
    EWrite nEEP_ADDR,[$A5]          ' Write the confirmation of writing in eeprom.
    Return
'===============================================================================
PNMRX108A: CData "PNMRX108,GGA,RMC,VTG",0       ' CKS = "14"    GGA + RMC
PNMRX103A: CData "PNMRX103,GGA,1,RMC,1,VTG,1",0 ' CKS = "02"    GGA + RMC + VTG 
PNMRX103B: CData "PNMRX103,GGA,1,RMC,1,VTG,0",0 ' CKS = "03"   GGA + RMC
PNMRX100A: CData "PNMRX100,0,9600.,0",0         ' CKS = "65"   set the UART baud rate To 9600 bps And no parity.
PNMRX100B: CData "PNMRX100,0,19200.,0",0        ' CKS = "50"   set the UART baud rate To 19200 bps And no parity.
'===============================================================================
' Online NMEA XOR checksum calculator: http://www.hhhh.org/wiml/proj/nmeaxor.html
XPNMRX108A: CData "$PNMRX108,GGA,RMC,VTG*14",13,10,0         ' CKS = "14"    GGA + RMC
XPNMRX103A: CData "$PNMRX103,GGA,1,RMC,1,VTG,1*02",13,10,0   ' CKS = "02"    GGA + RMC + VTG 
XPNMRX103B: CData "$PNMRX103,GGA,1,RMC,1,VTG,0*03",13,10,0   ' CKS = "03"   GGA + RMC
XPNMRX100A: CData "$PNMRX100,0,9600.,0*65",13,10,0           ' CKS = "65"   set the UART baud rate To 9600 bps And no parity.
XPNMRX100B: CData "$PNMRX100,0,19200.,0*50",13,10,0          ' CKS = "50"    set the UART baud rate To 19200 bps And no parity.
'===============================================================================
$endif      ' _GPS_Configuration_
'###############################################################################
_GPS_Main_Exit:      
'=============================================================================== 
' Init the GPS
    GoSub ExitClearAll
'=============================================================================== 
$endif      ' _AMICUS18_GPS_   
